# -*- coding: UTF-8 -*-
# vim: fdm=marker

__revision__ = '$Id$'

# Copyright © 2005-2009 Vasco Nunes, Piotr Ożarowski
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

# You may use and distribute this software under the terms of the
# GNU General Public License, version 2 or later


# XXX: keep stdlib, griffith.db and SQLAlchemy imports only in this file

import logging
import os.path
from gettext import gettext as _

from sqlalchemy import create_engine, or_, and_, not_, exists, asc, desc
from sqlalchemy.exc import OperationalError
from sqlalchemy.orm import sessionmaker, scoped_session
from sqlalchemy.sql.expression import Update, Delete

import db  # ORM data (SQLAlchemy stuff)
from gutils import get_filesystem_pagesize
from gutils import warning  # TODO: get rid of this import

log = logging.getLogger("Griffith")


class GriffithSQL(object):
    version = db.__version__
    DEFAULT_PORTS = dict(postgres=5432, mysql=3306, mssql=1433)

    def __init__(self, config, griffith_dir, fallback=True):
        #mapper = Session.mapper
        self.config = config
        self.data_dir = griffith_dir

        if config.get('type', None, section='database') is None:
            config.set('type', 'sqlite', section='database')

        if config.get('type', 'sqlite', section='database') != 'sqlite':
            if config.get('host', None, section='database') is None:
                config.set('host', '127.0.0.1', section='database')
            if config.get('user', None, section='database') is None:
                config.set('user', 'griffith', section='database')
            if config.get('passwd', None, section='database') is None:
                config.set('passwd', 'gRiFiTh', section='database')
            if config.get('name', None, section='database') is None:
                config.set('name', 'griffith', section='database')
            if config.get('port', 0, section='database') == 0:
                config.set('port', GriffithSQL.DEFAULT_PORTS[config.get('type', section='database')], section='database')

        conn_params = config.to_dict(section='database')
        conn_params.update({
            'port': int(conn_params.get('port', 0)),
            # SQLAlchemy 2.0: remove deprecated convert_unicode
            'engine_kwargs': {'echo': False},
        })

        # connect to database --------------------------------------{{{
        dbinitializingsql = None
        if config.get('type', section='database') == 'sqlite':
            sqlitefile = "%s.db" % os.path.join(griffith_dir, conn_params['name'])
            if not os.path.isfile(sqlitefile):
                # for new created database this is an optimization step syncing the database page size
                # to the filesystem page size
                dbinitializingsql = 'PRAGMA page_size=' + str(get_filesystem_pagesize(sqlitefile))
            url = "sqlite:///%s" % sqlitefile
        elif config.get('type', section='database') == 'postgres':
            # sqlalchemy version check because postgres dialect is renamed in sqlalchemy>=0.6 from postgres to postgresql
            from sqlalchemy import __version__ as sqlalchemyversion
            if list(map(int, sqlalchemyversion[:3].split('.'))) < [0, 6]:
                url = "postgres"
            else:
                url = "postgresql"
            url = url + "://%(user)s:%(passwd)s@%(host)s:%(port)d/%(name)s" % conn_params
        elif config.get('type', section='database') == 'mysql':
            conn_params['engine_kwargs']['pool_recycle'] = int(
                config.get('pool_recycle', 3600, section='database')
            )
            url = "mysql://%(user)s:%(passwd)s@%(host)s:%(port)d/%(name)s?charset=utf8&use_unicode=0" % conn_params
        elif config.get('type', section='database') == 'mssql':
            # use_scope_identity=0 have to be set as workaround for a sqlalchemy bug
            # but it is not guaranteed that the right identity value will be selected
            # because the select @@identity statement selects the very last id which
            # also can be a id from a trigger-insert or another user
            # sqlalchemy uses a wrong syntax. It has to select the id within the insert
            # statement: insert <table> (<columns>) values (<values>) select scope_identity()
            # (one statement !) After preparing and executing there should be a fetch
            # If it is executed as two separate statements the scope is lost after insert.
            url = "mssql://%(user)s:%(passwd)s@%(host)s:%(port)d/%(name)s?use_scope_identity=0" % conn_params
        else:
            config.set('type', 'sqlite', section='database')
            url = "sqlite:///%s.db" % os.path.join(griffith_dir, conn_params['name'])

        # try to establish a db connection
        try:
            engine = create_engine(url, **conn_params['engine_kwargs'])
            engine.connect()
            if dbinitializingsql is not None:
                # SQLAlchemy 2.0: use exec_driver_sql for driver-level PRAGMA
                with engine.begin() as conn:
                    conn.exec_driver_sql(dbinitializingsql)
        except Exception as e:    # InvalidRequestError, ImportError
            log.info("MetaData: %s", e)
            if not fallback:
                raise e
            config.set('type', 'sqlite', section='database')
            warning("%s\n\n%s" % (_('Cannot connect to database.\nFalling back to SQLite.'), _('Please check debug output for more informations.')))
            url = "sqlite:///%s.db" % os.path.join(griffith_dir, conn_params['name'])
            engine = create_engine(url)
            engine.connect()

        # scoped_session(...) is a workaround for unclosed sessions in the program
        # https://bugs.launchpad.net/griffith/+bug/574370
        self.Session = scoped_session(sessionmaker(bind=engine)) # create new sessions using this class
        self.engine = engine
        self.session = self.Session() # TODO: get rid of it, force developers to create new session using gsql.Session()
        #}}}

        # check if database needs an upgrade
        db.metadata.create_all(engine)
        try:
            v = self.session.query(db.Configuration).filter_by(param='version').first()    # returns None if table exists && param ISNULL
            if v is None:
                v = 0
            else:
                v = int(v.value)
        except OperationalError as e:
            log.info(e)
            v = 0
        except Exception as e:
            log.error(e)
            v = 0

        if v < self.version:
            from dbupgrade import upgrade_database
            if not upgrade_database(self, v, config):
                raise Exception('cannot upgrade database')
        elif v > self.version:
            log.error("database version mismatch (detected:%s; current:%s)", v, self.version)
            warning(_('This database requires newer version of Griffith.'))
            raise Exception("database version mismatch")

    def dispose(self):
        # close every session and connection so that a sqlite database file can be removed
        # otherwise it will remain opened by the current process and can't be deleted
        try:
            self.session.close()
            self.Session.close_all()
            self.engine.dispose()
            del self.session
            del self.Session
            del self.engine
        except Exception:
            log.exception('')


def update_whereclause(query, cond): # {{{
    if cond['loaned'] is True:
        query = query.where(db.Movie.loaned is True)
    if cond['loaned'] is False:
        query = query.where(db.Movie.loaned is False)
    if cond['seen'] is True:
        query = query.where(db.Movie.seen is True)
    if cond['seen'] is False:
        query = query.where(db.Movie.seen is False)

    if cond["collections"]:
        query = query.where(db.Movie.collection_id.in_(cond["collections"]))
    if cond["no_collections"]:
        query = query.where(or_(~db.Movie.collection_id.in_(cond["no_collections"]), db.Movie.collection_id is None))

    if cond["volumes"]:
        query = query.where(db.Movie.volume_id.in_(cond["volumes"]))
    if cond["no_volumes"]:
        query = query.where(or_(~db.Movie.volume_id.in_(cond["no_volumes"]), db.Movie.volume_id is None))

    loaned_to = []
    for per_id in cond["loaned_to"]:
        loaned_to.append(
            exists().where(
                and_(
                    db.Movie.movie_id == db.tables.loans.c.movie_id,
                    db.tables.loans.c.person_id == per_id,
                    db.tables.loans.c.return_date.is_(None),
                )
            )
        )
    if loaned_to:
        query = query.where(or_(*loaned_to))

    loan_history = []
    for per_id in cond["loan_history"]:
        loan_history.append(
            exists().where(
                and_(
                    db.Movie.movie_id == db.tables.loans.c.movie_id,
                    db.tables.loans.c.person_id == per_id,
                )
            )
        )
    if loan_history:
        query = query.where(or_(*loan_history))

    required_tags = []
    for tag_id in cond["required_tags"]:
        required_tags.append(
            exists().where(
                and_(
                    db.Movie.movie_id == db.MovieTag.movie_id,
                    db.MovieTag.tag_id == tag_id,
                )
            )
        )
    if required_tags:
        query = query.where(and_(*required_tags))

    tags = []
    for tag_id in cond["tags"]:
        tags.append(
            exists().where(
                and_(
                    db.Movie.movie_id == db.MovieTag.movie_id,
                    db.MovieTag.tag_id == tag_id,
                )
            )
        )
    if tags:
        query = query.where(or_(*tags))

    no_tags = []
    for tag_id in cond["no_tags"]:
        no_tags.append(
            ~exists().where(
                and_(
                    db.Movie.movie_id == db.MovieTag.movie_id,
                    db.MovieTag.tag_id == tag_id,
                )
            )
        )
    if no_tags:
        query = query.where(and_(*no_tags))

    for field in cond["equals_n"]:
        values = [db.tables.movies.columns[field] != value for value in cond["equals_n"][field]]
        query = query.where(and_(*values))

    for field in cond["startswith_n"]:
        values = [not_(db.tables.movies.columns[field].startswith(value)) for value in cond["startswith_n"][field]]
        query = query.where(and_(*values))

    for field in cond["like_n"]:
        values = [not_(db.tables.movies.columns[field].like(value)) for value in cond["like_n"][field]]
        query = query.where(and_(*values))

    for field in cond["contains_n"]: # XXX: it's not the SQLAlchemy's .contains() i.e. not for one-to-many or many-to-many collections
        values = [not_(db.tables.movies.columns[field].like('%' + value + '%')) for value in cond["contains_n"][field]]
        query = query.where(and_(*values))

    for field in cond["equals"]:
        values = [db.tables.movies.columns[field] == value for value in cond["equals"][field]]
        query = query.where(or_(*values))

    for field in cond["startswith"]:
        values = [db.tables.movies.columns[field].startswith(value) for value in cond["startswith"][field]]
        query = query.where(or_(*values))

    for field in cond["like"]:
        values = [db.tables.movies.columns[field].like(value) for value in cond["like"][field]]
        query = query.where(or_(*values))

    for field in cond["contains"]: # XXX: it's not the SQLAlchemy's .contains() i.e. not for one-to-many or many-to-many collections
        values = [db.tables.movies.columns[field].like('%' + value + '%') for value in cond["contains"][field]]
        query = query.where(or_(*values))

    # sorting
    if not isinstance(query, (Update, Delete)):
        for rule in cond.get('sort_by', []):
            if rule.endswith(" DESC"):
                reverse = True
                column = rule.replace(" DESC", '')
            else:
                column = rule.replace(" ASC", '') # note that " ASC" is optional
                reverse = False

            table = 'movies'
            tmp = column.split('.')
            if len(tmp) > 1:
                table = tmp[0]
                column = tmp[1]

            if reverse:
                query = query.order_by(desc(db.metadata.tables[table].columns[column]))
            else:
                query = query.order_by(asc(db.metadata.tables[table].columns[column]))

    log.debug(query)
    return query #}}}



# -*- coding: UTF-8 -*-

__revision__ = '$Id$'

# Copyright (c) 2009 Michael Jahn
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

# You may use and distribute this software under the terms of the
# GNU General Public License, version 2 or later

import gutils
from plugins.export import XmlExportBase
from gettext import gettext as _


class ExportPlugin(XmlExportBase):
    plugin_name         = "GCstar Database Export"
    plugin_description  = _("Full GCstar database export plugin")
    plugin_author       = "Michael Jahn"
    plugin_author_email = "griffith@griffith.cc"
    plugin_version      = "1.0"

    def __init__(self, database, locations, parent_window, search_conditions, config):
        XmlExportBase.__init__(self, database, locations, parent_window, search_conditions, config)
        self.config_section = 'export-gcstar'
        self.export_name    = 'GCstar database export'
        self.filename       = 'gcstar.gcs'
        self.encoding       = 'utf-8'
        self.true_value     = '1'
        self.false_value    = '0'

    def export_to_document(self, document, mainelement):
        collelement = document.createElement('collection')
        collelement.setAttribute('type', 'GCfilms')
        document.appendChild(collelement)
        comment = document.createComment('Griffith Export in GCstar database format')
        document.insertBefore(comment, collelement)
        XmlExportBase.export_to_document(self, document, collelement)
        collelement.setAttribute('items', str(self.exported_movies))

    def process_movie(self, document, mainelement, movie, keys):
        # create node
        itemelement = document.createElement('item')
        itemelement.setAttribute('id', '')
        itemelement.setAttribute('title', '')
        itemelement.setAttribute('date', '')
        itemelement.setAttribute('time', '')
        itemelement.setAttribute('director', '')
        itemelement.setAttribute('country', '')
        itemelement.setAttribute('genre', '')
        itemelement.setAttribute('image', '')
        itemelement.setAttribute('actors', '')
        itemelement.setAttribute('original', '')
        itemelement.setAttribute('webPage', '')
        itemelement.setAttribute('seen', '')
        itemelement.setAttribute('added', '')
        itemelement.setAttribute('format', '')
        itemelement.setAttribute('number', '')
        itemelement.setAttribute('identifier', '')
        itemelement.setAttribute('place', '')
        itemelement.setAttribute('rating', '')
        itemelement.setAttribute('audio', '')
        itemelement.setAttribute('subt', '')
        itemelement.setAttribute('age', '')
        itemelement.setAttribute('video', '')
        itemelement.setAttribute('serie', '')
        itemelement.setAttribute('rank', '')
        itemelement.setAttribute('trailer', '')
        itemelement.setAttribute('borrower', '')
        itemelement.setAttribute('lendDate', '')
        itemelement.setAttribute('borrowings', '')
        itemelement.setAttribute('favourite', '')
        itemelement.setAttribute('tags', '')
        mainelement.appendChild(itemelement)
        XmlExportBase.process_movie(self, document, itemelement, movie, keys)
        if not itemelement.getAttribute('title'):
            itemelement.setAttribute('title', itemelement.getAttribute('original'))

    def process_movie_value(self, document, dvdelement, key, value):
        if key == 'movies_number':
            dvdelement.setAttribute('id', value)
        elif key == 'movies_loaned':
            pass
        elif key == 'movies_seen':
            dvdelement.setAttribute('seen', value)
        elif key == 'movies_rating':
            dvdelement.setAttribute('rating', value)
        elif key == 'movies_color':             # SmallInteger
            pass
        elif key == 'movies_cond':              # SmallInteger
            pass
        elif key == 'movies_layers':            # SmallInteger
            pass
        elif key == 'movies_media_num':         # SmallInteger
            dvdelement.setAttribute('number', value)
        elif key == 'movies_runtime':           # Integer
            dvdelement.setAttribute('time', value)
        elif key == 'movies_year':              # year
            dvdelement.setAttribute('date', value)
        elif key == 'movies_o_title':           # VARCHAR
            dvdelement.setAttribute('original', value)
        elif key == 'movies_title':             # VARCHAR
            dvdelement.setAttribute('title', value)
        elif key == 'movies_director':          # VARCHAR
            dvdelement.setAttribute('director', value)
        elif key == 'movies_o_site':            # VARCHAR
            pass
        elif key == 'movies_site':              # VARCHAR
            dvdelement.setAttribute('webPage', value)
        elif key == 'movies_trailer':           # VARCHAR
            if len(value.strip()) > 0:
                trailersplit = value.split('file://')
                if len(trailersplit) > 1:
                    if gutils.is_windows_system():
                        dvdelement.setAttribute('trailer', trailersplit[1].lstrip('/'))
                    else:
                        dvdelement.setAttribute('trailer', trailersplit[1])
                else:
                    dvdelement.setAttribute('trailer', trailersplit[0])
        elif key == 'movies_country':           # VARCHAR
            dvdelement.setAttribute('country', value)
        elif key == 'movies_genre':             # VARCHAR
            dvdelement.setAttribute('genre', value.replace('\n', ''))
        elif key == 'movies_region':            # SmallInteger
            pass
        elif key == 'movies_studio':            # VARCHAR
            pass
        elif key == 'movies_classification':    # VARCHAR
            dvdelement.setAttribute('age', str(gutils.digits_only(value)))
        elif key == 'movies_cast':              # VARCHAR
            if len(value.strip()) > 0:
                actors = value.split('\n')
                actorsstring = ''
                delim = ''
                for actor in actors:
                    actorsplit = actor.split(_(' as '))
                    actornames = actorsplit[0]
                    role = ''
                    if len(actorsplit) > 1:
                        role = actorsplit[1]
                    actorsstring += delim + actornames.strip() + ' (' + role.strip() + ')'
                    delim = ', '
                dvdelement.setAttribute('actors', actorsstring)
        elif key == 'movies_plot':              # VARCHAR
            elem = document.createElement('synopsis')
            dvdelement.appendChild(elem)
            t = document.createTextNode(value)
            elem.appendChild(t)
        elif key == 'movies_notes':             # VARCHAR
            elem = document.createElement('comment')
            dvdelement.appendChild(elem)
            t = document.createTextNode(value)
            elem.appendChild(t)
        elif key == 'volumes_name':             # VARCHAR
            dvdelement.setAttribute('place', value)
        elif key == 'collections_name':         # VARCHAR
            dvdelement.setAttribute('serie', value)
        elif key == 'vcodecs_name':             # VARCHAR
            dvdelement.setAttribute('video', value)
        elif key == 'media_name':               # VARCHAR
            dvdelement.setAttribute('format', value)

    def process_movie_image(self, document, dvdelement, imagedata, imagemd5sum):
        if imagemd5sum and imagedata:
            dvdelement.setAttribute('image', self.filename + '_pictures/' + imagemd5sum + '.jpg')
            self.save_image_to_file(imagedata, imagemd5sum, self.filename + '_pictures/')

    def process_tags(self, document, dvdelement, tagsquery):
        tagsstring = ''
        delim = ''
        for tag in self.db.session.execute(tagsquery).fetchall():
            tagsstring += delim + self.convert_value(tag['name'])
            delim = ','
        dvdelement.setAttribute('tags', tagsstring)

    def process_languages(self, document, movieelement, languagesquery):
        audioelement = None
        for language in self.db.session.execute(languagesquery).fetchall():
            if audioelement is None:
                audioelement = document.createElement('audio')
                movieelement.appendChild(audioelement)
            lineelement = document.createElement('line')
            audioelement.appendChild(lineelement)
            colelement = document.createElement('col')
            lineelement.appendChild(colelement)
            t = document.createTextNode(self.convert_value(language['languages_name']))
            colelement.appendChild(t)
            colelement = document.createElement('col')
            lineelement.appendChild(colelement)
            t = document.createTextNode(self.convert_value(language['acodecs_name']))
            colelement.appendChild(t)

    def process_subtitles(self, document, movieelement, subtitlesquery):
        # process movie subtitles by query
        subtelement = None
        for subtitle in self.db.session.execute(subtitlesquery).fetchall():
            if subtelement is None:
                subtelement = document.createElement('subt')
                movieelement.appendChild(subtelement)
            lineelement = document.createElement('line')
            subtelement.appendChild(lineelement)
            colelement = document.createElement('col')
            lineelement.appendChild(colelement)
            t = document.createTextNode(self.convert_value(subtitle['name']))
            colelement.appendChild(t)



# -*- coding: UTF-8 -*-

__revision__ = '$Id$'

# Copyright (c) 2005-2009 Vasco Nunes, Piotr Ożarowski
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

# You may use and distribute this software under the terms of the
# GNU General Public License, version 2 or later

import datetime
import logging
import os.path
import zipfile
from io import StringIO
from shutil import rmtree, move
from tempfile import mkdtemp

from gi.repository import Gtk
from sqlalchemy import create_engine

import config
import gutils
import db
import sql
from initialize import dictionaries, people_treeview
from gutils import read_file
from gettext import gettext as _

try:
    pass
except Exception:
    pass

log = logging.getLogger('Griffith')


def create(self):
    """perform a compressed griffith database/posters/preferences backup"""
    #if self.db.session.bind.engine.name != 'sqlite':
    #    gutils.error(_("Backup function is available only for SQLite engine for now"), self.widgets['window'])
    #    return False

    default_name = "%s_backup_%s.zip" % (
        self.config.get('name', 'griffith', section='database'),
        datetime.date.isoformat(datetime.datetime.now()))
    filename = gutils.file_chooser(_("Save Griffith backup"),
        action=Gtk.FileChooserAction.SAVE, name=default_name,
        buttons=(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL, Gtk.STOCK_SAVE,
                 Gtk.ResponseType.OK))

    if filename and filename[0]:
        proceed = True
        zipfilename = filename[0]
        log.debug('Backup filename: %s', zipfilename)
        if os.path.isfile(zipfilename):
            if not gutils.question(_("File exists. Do you want to overwrite it?"), window=self.widgets['window']):
                proceed = False

        if proceed:
            try:
                if zipfile.zlib is not None:
                    log.debug('Creating zip file with compression')
                    mzip = zipfile.ZipFile(zipfilename, 'w', zipfile.ZIP_DEFLATED)
                else:
                    log.debug('Creating zip file without compression')
                    mzip = zipfile.ZipFile(zipfilename, 'w')
            except Exception:
                gutils.error(_("Error creating backup"), self.widgets['window'])
                return False
            log.debug('Preparing data and saving it to the zip archive')
            if self.db.session.bind.engine.name == 'sqlite':
                mzip.write(os.path.join(self.locations['home'], 'griffith.cfg').encode('utf-8'), arcname='griffith.cfg')
                db_file_name = "%s.db" % self.config.get('name', 'griffith', section='database')
                file_path = os.path.join(self.locations['home'], db_file_name).encode('utf-8')
                mzip.write(file_path, arcname=db_file_name)
            else:
                tmp_engine = None
                try:
                    tmp_dir = mkdtemp()
                    tmp_config_file = os.path.join(tmp_dir, 'griffith.cfg')
                    self.config.save(tmp_config_file)
                    tmp_config = config.Config(file=tmp_config_file)
                    tmp_config.set('type', 'sqlite', section='database')
                    tmp_config.set('file', 'griffith.db', section='database')
                    tmp_config.set('name', 'griffith', section='database')
                    tmp_config.save()
                    mzip.write(tmp_config._file, arcname='griffith.cfg')

                    tmp_file = os.path.join(tmp_dir, 'griffith.db')
                    tmp_engine = create_engine("sqlite:///%s" % tmp_file)
                    db.metadata.create_all(bind=tmp_engine)

                    # SQLite doesn't care about foreign keys much so we can just copy the data
                    with self.db.engine.connect() as src_conn, tmp_engine.begin() as tmp_conn:
                        for table in db.metadata.sorted_tables:
                            if table.name in ('posters', 'filters'):
                                continue  # see below
                            result = src_conn.execute(table.select())
                            data = result.fetchall()
                            if data:
                                tmp_conn.execute(table.insert(), data)

                    # posters
                    with self.db.engine.connect() as src_conn, tmp_engine.begin() as tmp_conn:
                        result = src_conn.execute(db.metadata.tables['posters'].select())
                        for poster in result:
                            tmp_conn.execute(
                                db.metadata.tables['posters'].insert(),
                                {
                                    'md5sum': poster.md5sum,
                                    'data': StringIO(poster.data).read(),
                                },
                            )

                    mzip.write(tmp_file, arcname='griffith.db')
                finally:
                    # disposing the temporary db connection before rmtree and in finally block to avoid locked db file
                    if tmp_engine:
                        tmp_engine.dispose()
                    rmtree(tmp_dir)
            gutils.info(_("Backup has been created"), self.widgets['window'])


@gutils.popup_message(_('Restoring database...'))
def copy_db(src_engine, dst_engine):
    log.debug('replacing old database with new one')
    db.metadata.drop_all(dst_engine)  # remove all previous data
    db.metadata.create_all(dst_engine)  # create table stucture

    # posters
    with src_engine.connect() as src_conn, dst_engine.begin() as dst_conn:
        result = src_conn.execute(db.metadata.tables['posters'].select())
        for poster in result:
            dst_conn.execute(
                db.metadata.tables['posters'].insert(),
                {'md5sum': poster.md5sum, 'data': poster.data},
            )

    with src_engine.connect() as src_conn, dst_engine.begin() as dst_conn:
        for table in db.metadata.sorted_tables:
            if table.name in ('posters',):
                continue  # see above
            log.debug('... processing %s table', table)
            result = src_conn.execute(table.select())
            data = [dict((col.key, row[col.name]) for col in table.c) for row in result]
            if data:
                log.debug('inserting new data...')
                insertcmd = table.insert()
                # insert in steps of 100 items because otherwise there is an error with mysql
                # I tried to insert more than 800 movies at ones: OperationalError
                for partition in range(0, len(data), 10):
                    dst_conn.execute(insertcmd, data[partition:partition + 10])

                if dst_engine.name in ('postgres', 'postgresql'):
                    # update current value of sequences
                    primary_column_name = list(table.primary_key.keys())[0]
                    if primary_column_name.endswith('_id'):
                        currval = max(row[primary_column_name] for row in data)
                        query = "SELECT setval('%s_%s_seq', %s)" % (table.name, primary_column_name, currval)
                        log.debug('updating sequence: %s', query)
                        try:
                            dst_conn.exec_driver_sql(query)
                        except Exception as e:
                            e = getattr(e, 'message', e)
                            log.error('... cannot update sequence: %s', e)


def merge_db(src_db, dst_db):  # FIXME
    merged = 0
    dst_db.session.rollback()  # cancel all pending operations
    src_session = src_db.Session()  # create new session
    dst_session = dst_db.Session()  # create new session
    src_session.query(db.Movie).count()
    for movie in src_session.query(db.Movie).all():
        if dst_session.query(db.Movie).filter_by(o_title=movie.o_title).first() is not None:
            continue
        t_movies = {}
        for column in list(movie.mapper.c.keys()):
            t_movies[column] = eval("movie.%s" % column)

        # replace number with new one
        t_movies["number"] = gutils.find_next_available(dst_db)

        # don't restore volume/collection/tag/language/loan data (it's dangerous)
        t_movies.pop('movie_id')
        t_movies.pop('loaned')
        t_movies.pop('volume_id')
        t_movies.pop('collection_id')

        if dst_db.add_movie(t_movies):  # FIXME
            print(t_movies)

        if movie.image is not None:
            # FIXME: where does self come from
            dest_file = os.path.join(self.locations['posters'], movie.image + '.jpg')
            if not os.path.isfile(dest_file):
                src_file = os.path.join(tmp_dir, movie.image + '.jpg')
                if os.path.isfile(src_file):
                    move(src_file, dest_file)
        merged += 1
    return merged


def restore(self, merge=False):
    """
    Merge database from:
    * compressed backup (*.zip)
    * SQLite2 *.gri file
    * SQLite3 *.db file
    """
    # let user select a backup file
    filename, path = gutils.file_chooser(
        _("Restore Griffith backup"),
        action=Gtk.FileChooserAction.OPEN, backup=True,
        buttons=(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
                 Gtk.STOCK_OPEN, Gtk.ResponseType.OK))
    if not filename:
        log.debug('no file selected')
        return False

    try:
        tmp_db = None
        tmp_dir = mkdtemp()
        os.mkdir(os.path.join(tmp_dir, 'posters'))
        print(filename)
        if filename.lower().endswith('.zip'):
            try:
                zip_file = zipfile.ZipFile(filename, 'r')
            except Exception as ex:
                gutils.error(_(f"Can't read backup file, {type(ex).__name__}: "
                                f"{ex}"), self.widgets['window'])
                return False

            old_config_file = False
            # unpack files to temporary directory
            for file_path in zip_file.namelist():
                file_name = os.path.split(file_path)[-1]
                if not os.path.isdir(file_name):
                    if not file_name:
                        log.debug('skipping %s', file_path)
                        continue

                    if 'posters' in file_path:
                        new_file = os.path.join(tmp_dir, 'posters', file_name)
                    else:
                        new_file = os.path.join(tmp_dir, file_name)
                    if file_name.endswith('.conf'):
                        old_config_file = new_file
                    with open(new_file, 'wb') as outfile:
                        outfile.write(zip_file.read(file_path))
            zip_file.close()

            # restore config file (new one will be created if old config
            # format is detected)
            tmp_config = config.Config(file=os.path.join(tmp_dir, 'griffith.cfg'))
            if old_config_file:
                log.info('Old config file detected. Please note that it will '
                         'not be used.')
                old_config_raw_data = read_file(old_config_file)
                if old_config_raw_data.find('griffith.gri') >= -1:
                    tmp_config.set('file', 'griffith.gri', section='database')

            # update filename var. to point to the unpacked database
            filename = os.path.join(tmp_dir, tmp_config.get('name', 'griffith', section='database') + '.db')
        else:  # not a zip file? prepare a fake config file then
            tmp_config = config.Config(file=os.path.join(tmp_dir, 'griffith.cfg'))
            tmp_config.set('type', 'sqlite', section='database')
            tmp_config.set('file', 'griffith.db', section='database')

        # prepare temporary GriffithSQL instance
        locations = {'home': tmp_dir}
        # check if file needs conversion
        if filename.lower().endswith('.gri'):
            from .dbupgrade import convert_from_old_db
            tmp_db = convert_from_old_db(tmp_config, filename, os.path.join(tmp_dir, 'griffith.db'), locations)
            if not tmp_db:
                log.info("MERGE: Can't convert database, aborting.")
                return False
        else:
            tmp_db = sql.GriffithSQL(tmp_config, tmp_dir, fallback=False)

        if merge:
            merge_db(tmp_db, self.db)
        else:
            self.db.session.rollback()  # cancel all pending operations
            copy_db(tmp_db.session.bind, self.db.session.bind)
            # update old database section with current config values
            # (important while restoring to external databases)
            for key in ('name', 'passwd', 'host', 'user', 'file', 'type', 'port'):
                tmp_config.set(key, self.config.get(key, section='database'), section='database')
            tmp_config._file = self.config._file
            self.config = tmp_config
            self.config.save()
            dictionaries(self)
            people_treeview(self)
            # let's refresh the treeview
            self.clear_details()
            self.populate_treeview()
        #gutils.info(_("Databases merged!\n\nProcessed movies: %s\nMerged movies: %s"%(movies, merged)), self.widgets['window'])
        gutils.info(_("Backup restored"), self.widgets['window'])
    except Exception:
        log.exception('')
        raise
    finally:
        # disposing the temporary db connection before rmtree and
        # in finally block to avoid locked db file
        if tmp_db:
            tmp_db.dispose()
        log.debug('temporary directory no logger needed, removing %s', tmp_dir)
        rmtree(tmp_dir)



# -*- coding: utf-8 -*-

__revision__ = '$Id$'

# Copyright (c) 2009

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

# You may use and distribute this software under the terms of the
# GNU General Public License, version 2 or later

from plugins.imp import ImportPlugin as IP
import gutils
from gettext import gettext as _

try:
    import pyodbc
except ImportError:
    pass

import logging
log = logging.getLogger("Griffith")


class ImportPlugin(IP):
    description  = 'MyDVDs (v1.6x)'
    author       = 'Michael Jahn'
    email        = 'griffith@griffith.cc'
    version      = '1.0'
    file_filters = '*.[mM][dD][bB]'
    mime_types   = None

    fileversion  = None
    connection   = None
    cursor       = None

    def initialize(self):
        if not IP.initialize(self):
            return False
        self.edit = False
        return True

    def set_source(self, name):
        IP.set_source(self, name)
        self.filename = name
        self.fileversion = self.read_fileversion()
        if self.fileversion is None:
            gutils.error(_('The format of the file is not supported.'))
            return False
        return True

    def count_movies(self):
        """Returns number of movies in file which is about to be imported"""
        count = 0
        if self.connection:
            try:
                curs = self.connection.cursor()
                curs.execute('SELECT COUNT(*) FROM dvd')
                count = curs.fetchone()[0]
            except Exception:
                log.exception('')
        else:
            log.error('MyDVDs Import: No connection object.')
        log.info('MyDVDs Import: %s movies for import' % count)
        return count

    def get_movie_details(self):
        """Returns dictionary with movie details"""
        if not self.cursor:
            self.cursor = self.connection.cursor()
            self.cursor.execute( \
                'SELECT id, title, year, rating, stars, director, synop, \
                        cover, runtime, genre, notes, trailer, star5 \
                 FROM dvd;')

        currentrow = self.cursor.fetchone()
        if not currentrow:
            return None

        details = {}
        try:
            if currentrow[1]:
                details['title'] = currentrow[1]
            if currentrow[2]:
                details['year'] = currentrow[2]
            if currentrow[3]:
                details['classification'] = currentrow[3]
            if currentrow[4]:
                details['cast'] = currentrow[4]
            if currentrow[5]:
                details['director'] = currentrow[5]
            if currentrow[6]:
                details['plot'] = currentrow[6]
            if currentrow[7]:
                details['poster'] = currentrow[7]
            if currentrow[8]:
                details['runtime'] = gutils.digits_only(currentrow[8])
            if currentrow[9]:
                genre = currentrow[9].replace('~', ', ')
                if len(genre) > 2:
                    details['genre'] = genre[2:]
            if currentrow[10]:
                details['notes'] = currentrow[10]
            if currentrow[11]:
                details['trailer'] = currentrow[11]
            if currentrow[12]:
                try:
                    details['rating'] = 2 * int(currentrow[12])
                except Exception:
                    pass
        except Exception:
            log.exception('')
            details = None
        return details

    def clear(self):
        """clear plugin before next source file"""
        IP.clear(self)
        if self.cursor:
            self.cursor.close()
            self.cursor = None
        if self.connection:
            self.connection.close()
            self.connection = None
        self.fileversion = None

    def destroy(self):
        """close all resources"""
        IP.destroy(self)

    def read_fileversion(self):
        version = None
        try:
            log.debug('Connecting with: DRIVER={Microsoft Access Driver (*.mdb)};DBQ=%s' % self.filename.replace('\\', '\\\\'))
            self.connection = pyodbc.connect('DRIVER={Microsoft Access Driver (*.mdb)};DBQ=%s' % self.filename.replace('\\', '\\\\'))
            curs = self.connection.cursor()
            tablerow = curs.tables(table='dvd').fetchone()
            if tablerow:
                version = 1.6
            curs.close()
        except Exception as e:
            log.error(str(e))
        log.info('MyDVDs Import: Found file version %s' % version)
        return version



# -*- coding: utf-8 -*-

__revision__ = '$Id: $'

# Copyright (c) 2011, 2012

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

# You may use and distribute this software under the terms of the
# GNU General Public License, version 2 or later

import logging
import os
import zipfile
from gettext import gettext as _
from shutil import rmtree
from tempfile import mkdtemp
from xml.dom import minidom, Node

import gutils
from plugins.imp import ImportPlugin as IP

log = logging.getLogger("Griffith")


class ImportPlugin(IP):
    description  = 'Tellico'
    author       = 'Elan Ruusamäe'
    email        = 'glen@delfi.ee'
    version      = '1.1'
    file_filters = '*.[tT][cC]'
    mime_types   = None

    fileversion  = None
    filedom      = None
    items        = None
    itemindex    = 0
    zipdir       = None

    def initialize(self):
        if not IP.initialize(self):
            return False
        self.edit = False
        return True

    def set_source(self, name):
        IP.set_source(self, name)
        self.filename = name
        self.fileversion = self.read_fileversion()
        if self.fileversion is None:
            gutils.error(_('The format of the file is not supported.'))
            return False
        return True

    def count_movies(self):
        """Returns number of movies in file which is about to be imported"""
        count = 0
        if self.filedom:
            try:
                collectionElement = self.filedom.getElementsByTagName('collection')[0]
                for element in collectionElement.childNodes:
                    if element.nodeType == Node.ELEMENT_NODE and element.nodeName == 'entry':
                        count = count + 1
                log.info('Tellico Import: %s movies for import' % count)
            except Exception:
                log.exception('')
        else:
            log.error('Tellico Import: No filedom object.')
        return count

    def get_movie_details(self):
        """Returns dictionary with movie details"""
        if not self.filedom:
            log.error('Tellico Import: filedom not open')
            return None

        if not self.items:
            collectionElement = self.filedom.getElementsByTagName('collection')[0]
            self.items = collectionElement.childNodes
            self.itemindex = 0
        if not self.items or len(self.items) < 1:
            return None
        if len(self.items) <= self.itemindex:
            return None
        item = self.items[self.itemindex]
        while not (item.nodeType == Node.ELEMENT_NODE and item.nodeName == 'entry') and len(self.items) > self.itemindex + 1:
            self.itemindex = self.itemindex + 1
            item = self.items[self.itemindex]
        if len(self.items) <= self.itemindex:
            return None
        if not len(item.childNodes):
            return None

        details = {}
        try:
            details['number'] = item.getAttribute('id')
            for node in item.childNodes:
                if node.nodeType == Node.ELEMENT_NODE and len(node.childNodes) > 0:
                    if node.nodeName == 'color':
                        value = node.childNodes[0].data.strip()
                        if value == 'Color':
                            details['color'] = 1
                        elif value == 'Black & White':
                            details['color'] = 2
                    elif node.nodeName == 'region':
                        details['region'] = node.childNodes[0].data.strip()
                    elif node.nodeName == 'running-time':
                        details['runtime'] = node.childNodes[0].data.strip()
                        try:
                            details['runtime'] = int(details['runtime'])
                        except Exception:
                            details['runtime'] = 0
                    elif node.nodeName == 'year':
                        details['year'] = node.childNodes[0].data.strip()
                    elif node.nodeName == 'orig-title':
                        details['o_title'] = node.childNodes[0].data.strip()
                    elif node.nodeName == 'title':
                        details['title'] = node.childNodes[0].data.strip()
                    elif node.nodeName == 'directors':
                        details['director'] = self._joinXmlNodes(node.getElementsByTagName('director'))
                    elif node.nodeName == 'writers':
                        details['screenplay'] = self._joinXmlNodes(node.getElementsByTagName('writer'))
                    elif node.nodeName == 'o_site':
                        details['o_site'] = node.childNodes[0].data.strip()
                    elif node.nodeName == 'url':
                        details['site'] = node.childNodes[0].data.strip()
                    elif node.nodeName == 'nationalitys':
                        details['country'] = self._joinXmlNodes(node.getElementsByTagName('nationality'))
                    elif node.nodeName == 'genres':
                        details['genre'] = self._joinXmlNodes(node.getElementsByTagName('genre'))
                    elif node.nodeName == 'certification':
                        details['classification'] = node.childNodes[0].data.strip()
                    elif node.nodeName == 'casts':
                        details['cast'] = self._joinCastNodes(node.getElementsByTagName('cast'), separator = "\n")
                    elif node.nodeName == 'plot':
                        details['plot'] = node.childNodes[0].data.strip()
                    elif node.nodeName == 'comments':
                        details['notes'] = node.childNodes[0].data.strip()
                    elif node.nodeName == 'cover':
                        imgfile = node.childNodes[0].data.strip()
                        # use 'image' field, 'poster' field is buggy, bug #913283
                        # https://bugs.launchpad.net/griffith/+bug/913283
                        details['image'] = os.path.join(self.zipdir, 'images', imgfile)
        except EOFError:
            details = None
        except Exception:
            log.exception('')
            details = None

        self.itemindex = self.itemindex + 1
        return details

    def clear(self):
        """clear plugin before next source file"""
        IP.clear(self)
        if self.filedom:
            rmtree(self.zipdir)
            self.zipdir = None
            self.filedom.unlink()
            self.filedom = None
            self.fileversion = None
            self.items = None
            self.itemindex = 0

    def destroy(self):
        """close all resources"""
        IP.destroy(self)

    def read_fileversion(self):
        version = None
        try:
            self.zipdir = self._extractZip(self.filename)
            self.filedom = minidom.parse(os.path.join(self.zipdir, 'tellico.xml'))
            rootElement = self.filedom.getElementsByTagName('tellico')[0]
            version = rootElement.getAttribute('syntaxVersion')
        except Exception as e:
            log.error(str(e))
            if self.filedom:
                self.filedom.unlink()
                self.filedom = None
        log.info('Tellico: Found file version %s' % version)
        return version

    def _extractZip(self, filename):
        zf = zipfile.ZipFile(filename, 'r')
        tmpdir = mkdtemp(prefix = 'griffit2tellico')
        zf.extractall(path = tmpdir)
        return tmpdir

    def _joinXmlNodes(self, elements, separator = ', '):
        ret = ''
        for element in elements:
            if len(element.childNodes) > 0:
                ret = ret + element.childNodes[0].data.strip() + separator

        # chop off last separator
        if len(elements) > 0:
            ret = ret[:-len(separator)]
        return ret

    # join nodes for cast
    def _joinCastNodes(self, elements, separator = ', '):
        """
        we assume that this format:
            <cast>
            <column>Actor/Actress</column>
            <column>Role</column>
            </cast>

        ideally the columns likely came from header, which may differ:
            <field flags="3" title="Cast" category="Cast" format="2" type="8" name="cast" description="...">
                <prop name="column1" >Actor/Actress</prop>
                <prop name="column2" >Role</prop>
                <prop name="columns" >2</prop>
            </field>
        """

        ret = ''
        for element in elements:
            actor = None
            columns = element.getElementsByTagName('column')
            actor = columns[0].childNodes[0].data.strip()
            if len(columns) > 1 and len(columns[1].childNodes) > 0:
                actor = actor + _(' as ') + columns[1].childNodes[0].data.strip()

            if actor:
                ret = ret + actor + separator

        # chop off last separator
        if len(elements) > 0:
            ret = ret[:-len(separator)]

        return ret



# -*- coding: UTF-8 -*-

__revision__ = '$Id$'

# Copyright (c) 2005-2011 Vasco Nunes, Piotr Ożarowski
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

# You may use and distribute this software under the terms of the
# GNU General Public License, version 2 or later

from gi.repository import Gtk
from gi.repository import GdkPixbuf
import version
import os
from pathlib import Path
import logging
from gutils import read_file
from gettext import gettext as _

log = logging.getLogger('Griffith')


def parse_translator_info(data):
    translator_credits = ""
    if data:
        for line in data.split('\n'):
            if line.startswith('* '):
                lang = line[2:]
                if _(lang) != lang:
                    line = "* %s:" % _(lang)
            translator_credits += "%s\n" % line
    else:
        translator_credits = _("See TRANSLATORS file")
    return translator_credits


class AboutDialog:
    """Shows a gtk about dialog"""

    def __init__(self, locations):
        # FIXME: is this statement still valid since Python 3 is used:
        #  "remember to encode this file in UTF-8"
        translators_file = Path(locations['share'], 'TRANSLATORS')
        translators_default_fname = Path('/usr/share/doc/griffith/TRANSLATORS')
        gpl2_fname = Path('/usr/share/common-licenses/GPL-2')
        images_dir = locations['images']

        def _open_url(new_dialog, link):
            from . import gutils
            gutils.run_browser(link)

        # Gtk.about_dialog_set_url_hook(_open_url)

        dialog = Gtk.AboutDialog()
        dialog.set_name(version.pname)
        dialog.set_version(version.pversion)
        dialog.set_copyright(
            "Copyright © 2005-2017 Vasco Nunes. Piotr Ożarowski")
        dialog.set_website(version.pwebsite)
        dialog.set_authors([
            _("Main Authors") + ':',
            version.pauthor.replace(', ', '\n') + "\n",
            _("Programmers") + ':',
            'Jessica Katharina Parth <Jessica.K.P@women-at-work.org>',
            'Michael Jahn <mikej06@hotmail.com>',
            'Ivo Nunes <netherblood@gmail.com>',
            'John Cheetham <kaama12@yahoo.co.uk>\n',
            _('Contributors') + ':',
            'Christian Sagmueller <christian@sagmueller.net>\n'
            'Arjen Schwarz <arjen.schwarz@gmail.com>\n'
            'Joshua Gentry\n'
            'Wolfgang Friebel'
            ])
        dialog.set_artists(
            [_("Logo, icon and general artwork by Peek <peekpt@gmail.com>."
               "\nPlease visit http://www.peekmambo.com/\n"),
             'seen / unseen icons by dragonskulle <dragonskulle@gmail.com>'
             ])
        data = None
        if translators_file.is_file():
            data = read_file(translators_file)
        elif translators_file.with_suffix('.gz').is_file():
            from .gutils import decompress
            data = decompress(read_file(translators_file.with_suffix('.gz')))
        elif os.name == 'posix':
            if translators_default_fname.is_file():
                data = read_file(translators_default_fname)
            elif translators_default_fname.with_suffix('.gz').is_file():
                from .gutils import decompress
                data = decompress(
                    read_file(translators_file.with_suffix('.gz')))
        dialog.set_translator_credits(parse_translator_info(data))
        logo_file = os.path.abspath(os.path.join(images_dir, 'griffith.png'))
        logo = GdkPixbuf.Pixbuf.new_from_file(logo_file)
        dialog.set_logo(logo)
        if gpl2_fname.is_file():
            dialog.set_license(read_file(gpl2_fname))
        else:
            gpl2_site = "https://www.gnu.org/licenses/old-licenses/gpl-2.0.html"
            dialog.set_license(
                _("This program is released under the GNUGeneral Public License.\n"
                                  f"Please visit {gpl2_site} for details."))
        dialog.set_comments(version.pdescription)
        dialog.connect("response", lambda d, r: d.destroy())

        # Add TMDB attribution (required condition of using the API)
        ca = dialog.get_content_area()
        logo_file2 = os.path.abspath(os.path.join(images_dir, 'tmdb.png'))
        logo2 = GdkPixbuf.Pixbuf.new_from_file(logo_file2)
        im = Gtk.Image.new_from_pixbuf(logo2)
        im.show()
        disclaimer = ("This product uses the TMDb API\nbut is not endorsed or "
                      "certified by TMDb.")
        lbl = Gtk.Label(disclaimer)
        lbl.show()
        ca.pack_end(lbl, True, True, 0)
        ca.pack_end(im, True, True, 5)

        dialog.show()



# -*- coding: utf-8 -*-

__revision__ = '$Id$'

# Copyright (c) 2009

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

# You may use and distribute this software under the terms of the
# GNU General Public License, version 2 or later

import logging
import os
from gettext import gettext as _
from xml.dom import minidom, Node

import gutils
from plugins.imp import ImportPlugin as IP

log = logging.getLogger("Griffith")


class ImportPlugin(IP):
    description  = 'eXtreme Movie Manager (v6.x / 7.x)'
    author       = 'Michael Jahn'
    email        = 'griffith@griffith.cc'
    version      = '1.0'
    file_filters = '*.[xX][mM][lL]'
    mime_types   = None

    fileversion  = None
    filedom      = None
    items        = None
    itemindex    = 0

    def initialize(self):
        if not IP.initialize(self):
            return False
        self.edit = False
        return True

    def set_source(self, name):
        IP.set_source(self, name)
        self.filename = name
        self.fileversion = self.read_fileversion()
        if self.fileversion is None:
            gutils.error(_('The format of the file is not supported.'))
            return False
        return True

    def count_movies(self):
        """Returns number of movies in file which is about to be imported"""
        count = 0
        if self.filedom:
            try:
                collectionElement = self.filedom.getElementsByTagName('XMM_Movie_Database')[0]
                for element in collectionElement.childNodes:
                    if element.nodeType == Node.ELEMENT_NODE and element.nodeName == 'Movie':
                        count = count + 1
                log.info('eXtreme Movie Manager Import: %s movies for import' % count)
            except Exception:
                log.exception('')
        else:
            log.error('eXtreme Movie Manager Import: No filedom object.')
        return count

    def get_movie_details(self):
        """Returns dictionary with movie details"""
        if not self.filedom:
            self.filedom = minidom.parse(self.filename)
        if not self.items:
            databaseElement = self.filedom.getElementsByTagName('XMM_Movie_Database')[0]
            self.items = databaseElement.childNodes
            self.itemindex = 0
        if not self.items or len(self.items) < 1:
            return None
        if len(self.items) <= self.itemindex:
            return None
        item = self.items[self.itemindex]
        while not (item.nodeType == Node.ELEMENT_NODE and item.nodeName == 'Movie') and len(self.items) > self.itemindex + 1:
            self.itemindex = self.itemindex + 1
            item = self.items[self.itemindex]
        if len(self.items) <= self.itemindex:
            return None
        if not len(item.childNodes):
            return None

        details = {}
        try:
            for node in item.childNodes:
                if node.nodeType == Node.ELEMENT_NODE:
                    if node.nodeName == 'Title' and len(node.childNodes) > 0:
                        details['title'] = node.childNodes[0].data.strip()
                    elif node.nodeName == 'OriginalTitle' and len(node.childNodes) > 0:
                        details['o_title'] = node.childNodes[0].data.strip()
                    elif node.nodeName == 'Year' and len(node.childNodes) > 0:
                        details['year'] = node.childNodes[0].data.strip()
                    elif node.nodeName == 'URL' and len(node.childNodes) > 0:
                        details['o_site'] = node.childNodes[0].data.strip()
                    elif node.nodeName == 'WebLinkScript' and len(node.childNodes) > 0:
                        details['site'] = node.childNodes[0].data.strip()
                    elif node.nodeName == 'TrailerFile1' and len(node.childNodes) > 0:
                        details['trailer'] = node.childNodes[0].data.strip()
                    elif node.nodeName == 'MPAA' and len(node.childNodes) > 0:
                        details['classification'] = node.childNodes[0].data.strip()
                    elif node.nodeName == 'Country' and len(node.childNodes) > 0:
                        details['country'] = node.childNodes[0].data.strip()
                    elif node.nodeName == 'Genre' and len(node.childNodes) > 0:
                        details['genre'] = node.childNodes[0].data.strip()
                    elif node.nodeName == 'Studio' and len(node.childNodes) > 0:
                        details['studio'] = node.childNodes[0].data.strip()
                    elif node.nodeName == 'UPC' and len(node.childNodes) > 0:
                        details['barcode'] = node.childNodes[0].data.strip()
                    elif node.nodeName == 'Region' and len(node.childNodes) > 0:
                        details['region'] = node.childNodes[0].data[0]
                    elif node.nodeName == 'Length' and len(node.childNodes) > 0:
                        details['runtime'] = node.childNodes[0].data.strip()
                        try:
                            details['runtime'] = int(details['runtime'])
                        except Exception:
                            details['runtime'] = 0
                    elif node.nodeName == 'Director' and len(node.childNodes) > 0:
                        details['director'] = node.childNodes[0].data.strip().replace('|', ', ')
                    elif node.nodeName == 'Writer' and len(node.childNodes) > 0:
                        details['screenplay'] = node.childNodes[0].data.strip().replace('|', ', ')
                    elif node.nodeName == 'Photographer' and len(node.childNodes) > 0:
                        details['cameraman'] = node.childNodes[0].data.strip().replace('|', ', ')
                    elif node.nodeName == 'Notes' and len(node.childNodes) > 0:
                        details['notes'] = node.childNodes[0].data.strip()
                    elif node.nodeName == 'Plot' and len(node.childNodes) > 0:
                        details['plot'] = node.childNodes[0].data.strip()
                    elif node.nodeName == 'Cover' and len(node.childNodes) > 0:
                        details['poster'] = os.path.join('images', node.childNodes[0].data.strip())
                    elif node.nodeName == 'Rating' and len(node.childNodes) > 0:
                        if 'rating' not in details:
                            try:
                                details['rating'] = int(node.childNodes[0].data)
                            except Exception:
                                details['rating'] = 0
                    elif node.nodeName == 'PersonalRating' and len(node.childNodes) > 0:
                        try:
                            details['rating'] = int(node.childNodes[0].data)
                        except Exception:
                            details['rating'] = 0
                    elif node.nodeName == 'Seen' and len(node.childNodes) > 0:
                        details['seen'] = (node.childNodes[0].data == 'True')
                    elif node.nodeName == 'Actors' and len(node.childNodes) > 0:
                        details['cast'] = ''
                        for actor in node.childNodes:
                            if actor.nodeType == Node.ELEMENT_NODE and actor.nodeName == 'Actor':
                                actorname = None
                                actorrole = None
                                for actorDetail in actor.childNodes:
                                    if actorDetail.nodeType == Node.ELEMENT_NODE and len(actorDetail.childNodes) > 0:
                                        if actorDetail.nodeName == 'ActorName':
                                            actorname = actorDetail.childNodes[0].data.strip()
                                        elif actorDetail.nodeName == 'ActorRole':
                                            actorrole = actorDetail.childNodes[0].data.strip()
                                if actorname:
                                    if actorrole:
                                        details['cast'] = details['cast'] + actorname + _(' as ') + actorrole + '\n'
                                    else:
                                        details['cast'] = details['cast'] + actorname + '\n'
                    elif node.nodeName == 'Disks' and len(node.childNodes) > 0:
                        try:
                            details['media_num'] = int(node.childNodes[0].data)
                        except Exception:
                            pass
                    elif node.nodeName == 'Media' and len(node.childNodes) > 0:
                        details['medium_id'] = node.childNodes[0].data
                    elif node.nodeName == 'Color' and len(node.childNodes) > 0:
                        if node.childNodes[0].data == 'False':
                            details['color'] = 2
                        else:
                            details['color'] = 1
        except EOFError:
            details = None
        except Exception:
            log.exception('')
            details = None
        self.itemindex = self.itemindex + 1
        return details

    def clear(self):
        """clear plugin before next source file"""
        IP.clear(self)
        if self.filedom:
            self.filedom.unlink()
            self.filedom = None
            self.fileversion = None
            self.items = None
            self.itemindex = 0

    def destroy(self):
        """close all resources"""
        self.clear()
        IP.destroy(self)

    def read_fileversion(self):
        version = None
        self.filedom = minidom.parse(self.filename)
        try:
            collectionElement = self.filedom.getElementsByTagName('XMM_Movie_Database')[0]
            movieElements = collectionElement.childNodes
            for movieElement in movieElements:
                if movieElement.nodeType == Node.ELEMENT_NODE and movieElement.nodeName == 'Movie':
                    version = 6.0
                    for movieDetailElement in movieElement.childNodes:
                        if movieDetailElement.nodeType == Node.ELEMENT_NODE and movieDetailElement.nodeName == 'TVShow':
                            version = 7.0
                            break
                    if version == 7.0:
                        break
        except Exception as e:
            log.error(str(e))
            self.filedom.unlink()
            self.filedom = None
        log.info('eXtreme Movie Manager Import: Found file version %s' % version)
        return version



# -*- coding: utf-8 -*-

__revision__ = '$Id$'

# Copyright (c) 2009

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

# You may use and distribute this software under the terms of the
# GNU General Public License, version 2 or later

from plugins.imp import ImportPlugin as IP
import gutils
from gettext import gettext as _

try:
    import sqlite3
except ImportError:
    from pysqlite3 import dbapi2 as sqlite3

import logging

log = logging.getLogger("Griffith")


class ImportPlugin(IP):
    description  = 'wintoolz Filmdatenbank (v1.0.0.x)'
    author       = 'Michael Jahn'
    email        = 'griffith@griffith.cc'
    version      = '1.0'
    file_filters = '*.[dD][bB]'
    mime_types   = None

    fileversion  = None
    connection   = None
    cursor       = None

    def initialize(self):
        if not IP.initialize(self):
            return False
        self.edit = False
        return True

    def set_source(self, name):
        IP.set_source(self, name)
        self.filename = name
        self.fileversion = self.read_fileversion()
        if self.fileversion is None:
            gutils.error(_('The format of the file is not supported.'))
            return False
        return True

    def count_movies(self):
        """Returns number of movies in file which is about to be imported"""
        count = 0
        if self.connection:
            try:
                c = self.connection.execute('SELECT COUNT(*) FROM Main')
                count = c.fetchone()[0]
            except Exception:
                log.exception('')
        else:
            log.error('wintoolz Filmdatenbank: No connection object.')
        log.info('wintoolz Filmdatenbank Import: %s movies for import' % count)
        return count

    def get_movie_details(self):
        """Returns dictionary with movie details"""
        if not self.cursor:
            self.cursor = self.connection.execute( \
                'SELECT Titel, Original, Produktionsjahr, Laufzeit,\
                 FSKZahl, Handlung, Ausstattung, EAN, Disks, Genre, Regie, Drehbuch,\
                 Kamera, Studio, Produktionsland, Darsteller, Rolle, Notiz, Cover,\
                 Mediendatei, BewertungHandlung, BewertungDarsteller, BewertungMusik,\
                 BewertungEmotionen, BewertungSpecialEffects, GesehenZahl, Farbdarstellung \
                 FROM Main')

        currentrow = self.cursor.fetchone()
        if not currentrow:
            return None

        details = {}
        try:
            if currentrow[0]:
                details['title'] = currentrow[0]
            if currentrow[1]:
                details['o_title'] = currentrow[1]
            if currentrow[2]:
                details['year'] = currentrow[2]
            if currentrow[3]:
                details['runtime'] = currentrow[3]
            if currentrow[4] and not currentrow[4] == 0:
                details['classification'] = currentrow[4]
            if currentrow[5]:
                details['plot'] = currentrow[5]
            if currentrow[6]:
                # setting medium_id to string mediumname; mapping is done in base class
                details['medium_id'] = currentrow[6]
            if currentrow[7]:
                details['barcode'] = currentrow[7]
            if currentrow[8]:
                details['media_num'] = currentrow[8]
            if currentrow[9]:
                details['genre'] = currentrow[9]
            if currentrow[10]:
                details['director'] = currentrow[10]
            if currentrow[11]:
                details['screenplay'] = currentrow[11]
            if currentrow[12]:
                details['cameraman'] = currentrow[12]
            if currentrow[13]:
                details['studio'] = currentrow[13]
            if currentrow[14]:
                details['country'] = currentrow[14]
            if currentrow[15] and currentrow[16]:
                cast = ''
                actors = currentrow[15].split('\r\n')
                roles = currentrow[16].split('\r\n')
                length = len(actors)
                if len(roles) > length:
                    length = len(roles)
                for index in range(0, length, 1):
                    actorrole = ''
                    if index < len(actors):
                        actorrole = actorrole + actors[index]
                    actorrole = actorrole + _(' as ')
                    if index < len(roles):
                        actorrole = actorrole + roles[index]
                    cast = cast + actorrole + '\n'
                details['cast'] = cast
            if currentrow[17]:
                details['notes'] = currentrow[17]
            if currentrow[18]:
                details['poster'] = currentrow[18]
            if currentrow[19]:
                details['trailer'] = currentrow[19]
            try:
                details['rating'] = round(0.4 * ( \
                    float(currentrow[20]) + float(currentrow[21]) + \
                    float(currentrow[22]) + float(currentrow[23]) + \
                    float(currentrow[24])), 0)
            except Exception:
                pass
            if currentrow[25]:
                details['seen'] = currentrow[25]
            if currentrow[26] and currentrow[26] == 'Schwarz-Weiss':
                details['color'] = 2
        except Exception:
            log.exception('')
            details = None

        return details

    def clear(self):
        """clear plugin before next source file"""
        IP.clear(self)
        self.cursor = None
        self.connection = None
        self.fileversion = None

    def destroy(self):
        """close all resources"""
        IP.destroy(self)

    def read_fileversion(self):
        version = None
        try:
            self.connection = sqlite3.connect(self.filename)
            if self.connection.execute('PRAGMA table_info(Main)').fetchone():
                version = 1.0
        except Exception as e:
            log.error(str(e))
        log.info('wintoolz Filmdatenbank Import: Found file version %s' % version)
        return version



# -*- coding: UTF-8 -*-

__revision__ = '$Id: PluginMovieIMDB.py 1660 2014-03-13 20:48:05Z mikej06 $'

# Copyright (©) 2005-2025 V.Nunes, P. Ożarowski, A. Flöter, W. Friebel
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

# You may use and distribute this software under the terms of the
# GNU General Public License, version 2 or later

import gutils
import movie
import re
from gettext import gettext as _
from bs4 import BeautifulSoup
from dataclasses import dataclass
import logging
import requests

log = logging.getLogger('Griffith')

plugin_name = 'IMDb'
plugin_description = 'Internet Movie Database'
plugin_url = 'www.imdb.com'
plugin_language = _('English')
plugin_author = 'Vasco Nunes, Piotr Ożarowski, Andreas Flöter, W. Friebel'
plugin_author_email = 'wp.friebel@gmail.com'
plugin_version = '1.22'


def before_more(data):
    for element in ['>See more<', '>more<', '>Full summary<', '>Full synopsis<']:
        tmp = data.find(element)
        if tmp > 0:
            data = data[:tmp] + '>'
    return data


def read_page(filename):
    with open(filename, 'r') as fp:
        return fp.read()


def is_genres_span(tag):
    return tag.name == 'span' and tag.text.strip() == 'Genres'


def is_director_span(tag):
    return tag.name == 'span' and tag.text.strip() == 'Director'


def is_language_tag(tag):
    # if tag.name == 'a' and '/search/title?title_type=feature&primary_language' in tag['href']:
    #     print(f"tag.name={tag.name}, tag['href']={tag['href']}")
    return tag.name == 'a' and '/search/title?title_type=feature&primary_language' in tag['href']


def is_mediaviewer_tag(tag):
    # if tag.name == 'a' and 'mediaviewer' in tag['href']:
    #     print(f"tag.name={tag.name}, tag['href']={tag['href']}")
    return tag.name == 'a' and 'mediaviewer' in tag['href']


def is_storyline_tag(tag):
    # <li class="ipc-metadata-list__item ipc-metadata-list-item--link" data-testid="storyline-certificate" role="presentation">
    # if (tag.name == 'li' and 'data-testid' in tag.attrs.keys()
    #         and 'storyline-certificate' in tag.attrs['data-testid']):
    #     print(f"tag.name={tag.name} {tag.attrs}")
    return (tag.name == 'li' and 'data-testid' in tag.attrs.keys()
            and 'storyline-certificate' in tag.attrs['data-testid'])


def is_studio_tag(tag):
    # <li role="presentation" class="ipc-metadata-list__item ipc-metadata-list-item--link" data-testid="title-details-companies">
    # if tag.name == 'li' and 'data-testid' in tag.attrs.keys()
    #     and 'title-details-companies' in tag.attrs['data-testid']:
    #     print(f"tag.name={tag.name} {tag.attrs}")
    return (tag.name == 'li' and 'data-testid' in tag.attrs.keys()
            and 'title-details-companies' in tag.attrs['data-testid'])


def is_plot_tag(tag):
    return (tag.name == 'div' and 'data-testid' in tag.attrs.keys()
            and 'sub-section-summaries' in tag.attrs['data-testid'])


@dataclass
class MovieInfo:
    title: str
    movie_id: str
    extra_info: str


class Plugin(movie.Movie):
    def __init__(self, movie_id):
        self.page_main = None
        self.page_cast = None
        self.page_plot = None
        self.encode = 'utf-8'
        self.movie_id = movie_id
        self.url = f"https://www.imdb.com/title/tt{self.movie_id}/"
        self.soup = None
        self.soup_cast = None

    def initialize(self):
        self.page_cast = self.open_page(url=self.url + 'fullcredits')
        self.page_main = self.open_page(url=self.url)
        self.page_plot = self.open_page(url=self.url + 'plotsummary')
        # self.comp_page = self.open_page(url=self.url + 'companycredits')
        # self.tagl_page = self.open_page(url=self.url + 'taglines')
        # self.cert_page = self.open_page(url=self.url + 'parentalguide')
        # self.release_page = self.open_page(url=self.url + 'releaseinfo')
        self.soup = BeautifulSoup(self.page_main, 'html.parser')
        self.soup_cast = BeautifulSoup(self.page_cast, 'html.parser')
        self.soup_plot = BeautifulSoup(self.page_plot, 'html.parser')

    def get_image(self):
        url_pattern = re.compile(r'^((https://www.imdb.com)?/title/tt\d+/mediaviewer/rm\d+/)(\?ref_=tt_ov_i)?$')
        self.image_url = ''
        image_page_url = ''

        for tag in self.soup.find_all(is_mediaviewer_tag):
            match = url_pattern.match(tag['href'])
            if match:
                image_page_url = match.groups()[0]
                log.debug(f"URL candidate: {match.groups()[0]}")
                if match.groups()[2]:
                    if not match.groups()[1]:
                        image_page_url = f"https://www.imdb.com{image_page_url}"
                    # self.image_url = image_page_url
                    log.debug(f"PluginMovieIMDB: Image Page URL: {image_page_url}")
                    break
                else:
                    log.debug(f"Not correct URL found: {match.groups()[0]}")
        image_page = requests.get(image_page_url,
                                  headers=self.config.http_header)
        if image_page.status_code != 200:
            log.debug(f"Unable to retrieve image page: {image_page_url}, "
                      f"code:{image_page.status_code}")
            return
        soup_image_page = BeautifulSoup(image_page.content, 'html.parser')
        log.debug(f"soup_image_page: {len(soup_image_page.body)}")
        # <img alt="Gwyneth Paltrow and Joseph Fiennes in Shakespeare in Love (1998)"
        # class="sc-7c0a9e7c-0 eWmrns" data-image-id="rm381097216-curr"
        # sizes="100vw" src="https://m.media-amazon.com/images/M/MV5BM2ZkNjM5M
        # jEtNTBlMC00OTI5LTgyYmEtZDljMzNmNzhiNzY0XkEyXkFqcGdeQXVyNDYyMDk5MTU@.
        # _V1_.jpg"
        for img_tag in soup_image_page.find_all('img', {'class': 'sc-7c0a9e7c-0'}):
            image_url = img_tag['src']
            log.debug(f"image_url: {image_url}")
            if image_url.endswith('.jpg'):
                self.image_url = image_url
                log.debug(f"Found image_url: {image_url}")
            break

    def get_o_title(self):
        self.o_title = gutils.trim(self.page_main, '<div class="originalTitle">', '<span')
        if not self.o_title:
            self.o_title = gutils.regextrim(self.page_main, '<h1 itemprop="name"[^>]*>', '&nbsp;')
        if not self.o_title:
            self.o_title = gutils.trim(self.page_main, 'og:title\' content="', '"')
        if not self.o_title:
            self.o_title = re.sub(' [(].*', '', gutils.trim(self.page_main, '<title>', '</title>'))
        self.o_title = gutils.clean(re.sub('"', '', self.o_title))

    def get_title(self):
        title_section = self.soup.find('h1', {'data-testid': 'hero__pageTitle'})
        if title_section:
            title = title_section.find('span', {'class': 'hero__primary-text'})
            if title:
                self.title = title.text.strip()
            else:
                self.title = ''

    def get_director(self):
        self.director = ''
        directors = []
        div_tag = None
        for tag in self.soup.find_all(is_director_span):
            div_tag = tag.next_sibling
            log.debug(f"div_tag: {div_tag}")
            break
        if div_tag:
            for a_tag in div_tag.find_all(
                    'a',
                    {'class': 'ipc-metadata-list-item__list-content-item '
                              'ipc-metadata-list-item__list-content-item--link'}):
                directors.append(a_tag.text.strip())
            if directors:
                self.director = ', '.join(directors)

    def get_plot(self):
        self.plot = ''
        plot_summaries = []
        div_summaries_tag = self.soup_plot.find(is_plot_tag)
        if div_summaries_tag:
            for div_tag in div_summaries_tag.find_all(
                    'div',
                    {'class': 'ipc-html-content-inner-div'}):
                plot_summary = div_tag.find('div', {'role': 'presentation'})
                if (plot_summary):
                    plot_summary = plot_summary.text.strip()
                    plot_summary = plot_summary.replace(r"\n", ' ')
                    pos_author = plot_summary.find('—')
                    plot_summary = plot_summary[:pos_author]
                    a_author_tag = div_tag.find('a', {'class': 'ipc-link ipc-link--base'})
                    if a_author_tag:
                        author = a_author_tag.text.strip()
                        plot_summaries.append(f"{plot_summary}\nAuthor: {author}")
                        log.debug(f"{plot_summary}\nAuthor: {author}")
                    else:
                        log.debug("No author 'a' tag found")
                        plot_summaries.append(f"{plot_summary}")
                        log.debug(f"{plot_summary}")
            self.plot = '\n\n'.join(plot_summaries)

    def get_year(self):
        self.year = gutils.trim(self.page_main, '<a href="/year/', '</a>')
        self.year = gutils.after(self.year, '>')
        if not self.year:
            tmp = gutils.trim(self.page_main, '<title>', '</title>')
            tmp = re.search('([0-9]{4})', tmp)
            if tmp:
                self.year = tmp.group(0)

    def get_runtime(self):
        runtime_hm_pattern = re.compile(r'(\d+).*hour\D+(\d+)\D+minutes')
        runtime_h_pattern = re.compile(r'(\d+).*hour\D*')
        hours, minutes = 0, 0

        # <div data-testid="title-techspecs-section" class="sc-f65f65be-0 bBlII">
        tec_section = self.soup.find('div', {'data-testid': 'title-techspecs-section'})
        # print(f"\ntec_section:\n{tec_section}")
        if tec_section:
            # <div class="ipc-metadata-list-item__content-container">1<!-- --> <!-- -->hour<!-- --> <!-- -->55<!-- --> <!-- -->minutes</div>
            tec_topics = tec_section.find_all(
                'div', {'class': 'ipc-metadata-list-item__content-container'})
            # print(f"\ntec_topics:\n{tec_topics}")
            for tec_topic in tec_topics:
                match = runtime_hm_pattern.match(tec_topic.text)
                # print(f"_____ match:\n{match}")
                if match:
                    hours = int(match.groups()[0])
                    minutes = int(match.groups()[1])
                    break
                else:
                    # print(f"tech_topic={tec_topic.text}")
                    match = runtime_h_pattern.match(tec_topic.text)
                    if match:
                        # print(f"________ match={match}, {match.groups()[0]}")
                        hours = int(match.groups()[0])
                        break
        if hours > 0 or (hours == 0 and minutes > 0):
            self.runtime = hours * 60 + minutes

    def get_genre(self):
        self.genre = ''
        genres = ''
        pattern = re.compile('^.+"genre":\[([^]]+)].+$', re.DOTALL)
        match = pattern.match(self.page)
        if match:
            genres = match.groups()[0]
            genres = genres.replace('"', '')
            genres = ', '.join(genres.split(','))
            self.genre = genres

    def get_cast(self):
        self.cast = ''
        self.cast = gutils.trim(self.page_cast, '<table class="cast_list">', '</table>')
        if self.cast == '':
            self.cast = gutils.trim(self.page_main, '<table class="cast">', '</table>')
        self.cast = self.cast.replace(' ... ', _(' as '))
        self.cast = self.cast.replace('...', _(' as '))
        self.cast = self.cast.replace("\n", '')
        self.cast = self.cast.replace('</tr>', "\n")
        self.cast = before_more(self.cast)
        self.cast = gutils.clean(self.cast)
        self.cast = re.sub('[ \t]+', ' ', self.cast)
        self.cast = re.sub('( *\n *)+', '\n', self.cast)
        # self.cast = re.sub(' *\n\n *', '\n', self.cast)
        self.cast = self.cast.strip()

    def get_classification(self):
        """
        The url provides the classification in one or more countries. It would
        be a lot more effort to parse this page since the country to be used
        must be defined and a fallback if no country is defined or the country
        which is defined is not in the list of classifications.
        """
        self.classification = ''

    def get_studio(self):
        studio_list = []
        self.studio = ''
        studio_tag = self.soup.find(is_studio_tag)
        if studio_tag:
            studios = studio_tag.find_all(
                'a',
                {'class': "ipc-metadata-list-item__list-content-item "
                          "ipc-metadata-list-item__list-content-item--link"})
            for studio in studios:
                # print(f"studio.text={studio.text}")
                if studio.text.strip():
                    studio_list.append(studio.text.strip())
        self.studio = ', '.join(studio_list)

    def get_o_site(self):
        self.o_site = ''

    def get_site(self):
        self.site = f"https://www.imdb.com/title/tt{self.movie_id}/"

    def get_trailer(self):
        """
        Sets the trailer URL but does not check if actually trailers exist.
        """
        self.trailer = f"https://www.imdb.com/title/tt{self.movie_id}/trailers"

    def get_country(self):
        """not used"""
        self.country = gutils.after(gutils.trim(self.page_main, 'country_of_origin', '</a>'), '>')

    def get_rating(self):
        rating = gutils.strip_tags(gutils.trim(self.page_main, 'IMDb RATING', '</span>'))
        if rating:
            try:
                # self.rating = round(float(rating), 0)
                self.rating = float(rating)
            except ValueError as _exc:
                self.rating = 0
        else:
            self.rating = 0

    def get_notes(self):
        self.notes = ''
        color_value = ''
        sound_value = []
        language_value = ''

        a_language_tag = self.soup.find(is_language_tag)
        if a_language_tag:
            language_value = a_language_tag.text
        language = language_value

        # <div data-testid="title-techspecs-section" class="sc-f65f65be-0 bBlII">
        tec_section = self.soup.find('div', {'data-testid': 'title-techspecs-section'})
        # print(f"\ntec_section:\n{tec_section}")
        if tec_section:
            # <a aria-disabled="false" class="ipc-metadata-list-item__list-content-item ipc-metadata-list-item__list-content-item--link" href="/search/title/?colors=color&amp;ref_=tt_spec_att" role="button" tabindex="0">Color</a>
            a_tags = tec_section.find_all(
                'a', {'class': r'ipc-metadata-list-item__list-content-item ipc-metadata-list-item__list-content-item--link'})
            for a_tag in a_tags:
                if 'colors=' in a_tag['href']:
                    # href="/search/title/?colors=color&ref_=tt_spec_att"
                    color_value = a_tag.text.strip()
                if 'sound_mixes=' in a_tag['href']:
                    # href="/search/title/?sound_mixes=stereo&ref_=tt_spec_att"
                    sound_value.append(a_tag.text.strip())
        color = color_value
        sound = ', '.join(sound_value)

        tagline = ''
        note_list = []
        if len(language) > 0:
            note_list.append("%s: %s" % (_('Language'), language))
        if len(sound) > 0:
            note_list.append("%s: %s" % (gutils.strip_tags(_('<b>Audio</b>')), sound))
        if len(color) > 0:
            note_list.append("%s: %s" % (_('Color'), color))
        if len(tagline) > 0:
            note_list.append("%s: %s" % ('Tagline', tagline))
        self.notes = '\n'.join(note_list)

    def get_screenplay(self):
        self.screenplay = ''
        # parts = re.split('<a href=', gutils.trim(self.page_cast, '>Writing Credits', '</table>'))
        # if len(parts) > 1:
        #     for part in parts[1:]:
        #         screenplay = gutils.trim(part, '>', '<')
        #         screenplay = screenplay.replace('\n', '')
        #         screenplay = screenplay.strip()
        #         if screenplay == 'WGA':
        #             continue
        #         screenplay = screenplay.replace(' (written by)', '')
        #         screenplay = screenplay.replace(' and<', '<')
        #         if screenplay not in self.screenplay:
        #             self.screenplay = self.screenplay + screenplay + ', '
        #     if len(self.screenplay) > 2:
        #         self.screenplay = self.screenplay[0:len(self.screenplay) - 2]
        #         self.screenplay = re.sub('[ \t]+', ' ', self.screenplay)

    def get_cameraman(self):
        """not used"""
        self.cameraman = ''
        # tmp = gutils.regextrim(self.page_cast, '>Cinematography by', '</table>')
        # tmp = tmp.split('href="')
        # if len(tmp) > 1:
        #     for entry in tmp[1:]:
        #         entry = gutils.trim(entry, '>', '<')
        #         entry = entry.replace('\n', '')
        #         entry = entry.strip()
        #         if entry:
        #             self.cameraman = self.cameraman + entry + ', '
        #     if self.cameraman:
        #         self.cameraman = self.cameraman[:-2]


class SearchPlugin(movie.SearchMovie):

    def __init__(self):
        """
        https://www.imdb.com/find?s=tt;q=
        Seems to give the best results. 25 results for "Damages", popular titles
        first.
        """
        super().__init__()
        self.original_url_search = 'https://www.imdb.com/find?s=tt&q='
        self.translated_url_search = 'https://www.imdb.com/find?s=tt&q='
        self.encode = 'utf8'
        self.page = None

    def search(self, parent_window):
        if not self.open_search(parent_window):
            return None
        return self.page

    def get_searches(self):
        movie_id_pattern = re.compile(r'/title/tt(\d+).*$')
        soup = BeautifulSoup(self.page, 'html.parser')
        # <div class="ipc-metadata-list-summary-item__tc">
        movie_div_items = soup.find_all(
            'div',
            {'class': 'ipc-metadata-list-summary-item__tc'})
        for movie_item in movie_div_items:
            # <div class="ipc-metadata-list-summary-item__tc">
            movie_entry = movie_item.find(
                'a', {'class': 'ipc-metadata-list-summary-item__t'})
            movie_title = movie_entry.text.strip()
            movie_id = movie_entry['href']
            match = movie_id_pattern.match(movie_id)
            if match:
                movie_id = match.groups()[0]
            else:
                movie_id = ''
            # <li role="presentation" class="ipc-inline-list__item">
            movie_additional_infos = movie_item.find_all(
                'li', {'class': 'ipc-inline-list__item'})
            infos = []
            for additional_info in movie_additional_infos:
                # <span class="ipc-metadata-list-summary-item__li" aria-disabled="false">
                info = additional_info.find(
                    'span', {'class': 'ipc-metadata-list-summary-item__li'})
                if info:
                    infos.append(info.text)
            movie_info = MovieInfo(movie_title, movie_id, '|'.join(infos))
            self.ids.append(movie_info.movie_id)
            self.titles.append(' - '.join((movie_info.title,
                                           movie_info.extra_info)))
            # log.debug(f"movie={movie}\n____")


class SearchPluginTest(SearchPlugin):
    """
    Plugin test for the search plugin
    Configuration for automated tests:
    dict { movie_id -> [ expected result count for original url,
                         expected result count for translated url ] }
    """
    test_configuration = {
        'Damages': [25, 25],
        'Ein glückliches Jahr': [17, 17],
        'Shakespeare in Love': [25, 25],
        }


class PluginTest:
    """
    Plugin test for the extraction of content for a film
    Configuration for automated tests:
    dict { movie_id -> dict { arribute -> value } }

    value: * True/False if attribute only should be tested for any value
           * or the expected value
    """
    test_configuration = {
        '0138097': {
            'title': 'Shakespeare in Love',
            'o_title': 'Shakespeare in Love',
            'director': 'John Madden',
            'plot': True,
            'cast': """Geoffrey Rush as Philip Henslowe
Tom Wilkinson as Hugh Fennyman
Steven O'Donnell as Lambert
Tim McMullan as Frees (as Tim McMullen)
Joseph Fiennes as Will Shakespeare
Steven Beard as Makepeace, the Preacher
Antony Sher as Dr Moth
Patrick Barlow as Will Kempe
Martin Clunes as Richard Burbage
Sandra Reinton as Rosaline
Simon Callow as Tilney, Master of the Revels
Judi Dench as Queen Elizabeth
Bridget McConnell as Lady in Waiting (as Bridget McConnel)
Georgie Glen as Lady in Waiting
Nicholas Boulton as Henry Condell
Gwyneth Paltrow as Viola De Lesseps
Imelda Staunton as Nurse
Colin Firth as Lord Wessex
Desmond McNamara as Crier
Barnaby Kay as Nol
Jim Carter as Ralph Bashford
Paul Bigley as Peter, the Stage Manager
Jason Round as Actor in Tavern
Rupert Farley as Barman
Adam Barker as First Auditionee
Joe Roberts as John Webster
Harry Gostelow as Second Auditionee
Alan Cody as Third Auditionee
Mark Williams as Wabash
David Curtiz as John Hemmings
Gregor Truter as James Hemmings
Simon Day as First Boatman
Jill Baker as Lady De Lesseps
Amber Glossop as Scullery Maid
Robin Davies as Master Plum
Hywel Simons as Servant
Nicholas Le Prevost as Sir Robert De Lesseps
Ben Affleck as Ned Alleyn
Timothy Kightley as Edward Pope
Mark Saban as Augustine Philips
Bob Barrett as George Bryan
Roger Morlidge as James Armitage
Daniel Brocklebank as Sam Gosse
Roger Frost as Second Boatman
Rebecca Charles as Chambermaid
Richard Gold as Lord in Waiting
Rachel Clarke as First Whore
Lucy Speed as Second Whore
Patricia Potter as Third Whore
John Ramm as Makepeace's Neighbour
Martin Neely as Paris / Lady Montague (as Martin Neeley)
The Choir of St. George's School in Windsor as (as The Choir of St George's School Windsor)
Rest of cast listed alphabetically:
Jason Anthony as Nobleman (uncredited)
Steve Apelt as Theatregoer (uncredited)
Paul Bannon as Theatre Attendee (uncredited)
Matthew Christian as Queen Elizabeth I Party Guest (uncredited)
Kelley Costigan as Theatregoer (uncredited)
Matthew Crosby as Theatregoer (uncredited)
Rupert Everett as Christopher Marlowe (uncredited)
Russell Greening as Thomas Crown (uncredited)
John Inman as Lady Capulet in Play (uncredited)
Carlisle Redthunder as Indian (uncredited)""",
            'country': 'United States',
            'genre': 'Comedy, Drama, History',
            'classification': False,
            'studio': 'Universal Pictures, Miramax, The Bedford Falls Company',
            'o_site': False,
            'site': 'https://www.imdb.com/title/tt0138097/',
            'trailer': 'https://www.imdb.com/title/tt0138097/trailers',
            'year': 1998,
            'notes': _('Language') + ': English\n'
                     + _('Audio') + ': Dolby Digital, DTS, SDDS\n'
                     + _('Color') + ': Color',
#             'notes': _('Language') + ': English\n'
#                      + _('Audio') + ': Dolby Digital, DTS, SDDS\n'
#                      + _('Color') + ': Color\n\
# Tagline: ...A Comedy About the Greatest Love Story Almost Never Told...\n\
# Love is the only inspiration',
            'runtime': 123,
            'image': True,
            'rating': 7.0,
            # 'screenplay': 'Marc Norman, Tom Stoppard',
            'screenplay': '',
            # 'cameraman': 'Richard Greatrex',
            'cameraman': '',
            'barcode': False
            },
        }



# -*- coding: UTF-8 -*-

__revision__ = '$Id$'

# Copyright (c) 2005-2007 Vasco Nunes
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

# You may use and distribute this software under the terms of the
# GNU General Public License, version 2 or later

import csv
from gi.repository import Gtk
import os
import gutils
from plugins.export import Base
from gettext import gettext as _


class ExportPlugin(Base):
    name = "CSV"
    description = _("Full CSV list export plugin")
    author = "Vasco Nunes"
    email = "<vasco.m.nunes@gmail.com>"
    version = "0.3"

    fields_to_export = ('number', 'o_title', 'title', 'director', 'year', 'classification', 'country',
                        'genre', 'rating', 'runtime', 'studio', 'seen', 'loaned', 'o_site', 'site', 'trailer',
                        'plot', 'cast', 'notes', 'image', 'volumes.name', 'collections.name', 'media.name',
                        'screenplay', 'cameraman', 'barcode', 'color', 'cond', 'layers', 'region',
                        'media_num', 'vcodecs.name')

    def run(self):
        basedir = None
        if self.config is not None:
            basedir = self.config.get('export_dir', None, section='export-csv')
        if not basedir:
            filename = gutils.file_chooser(_("Export a %s document")%"CSV", action=Gtk.FileChooserAction.SAVE, \
                buttons=(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL, Gtk.STOCK_SAVE, Gtk.ResponseType.OK), name='griffith_list.csv')
        else:
            filename = gutils.file_chooser(_("Export a %s document")%"CSV", action=Gtk.FileChooserAction.SAVE, \
                buttons=(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL, Gtk.STOCK_SAVE,Gtk.ResponseType.OK), name='griffith_list.csv', folder=basedir)
        if filename and filename[0]:
            if self.config is not None and filename[1]:
                self.config.set('export_dir', filename[1], section='export-csv')
                self.config.save()
            overwrite = None
            if os.path.isfile(filename[0]):
                if gutils.question(_("File exists. Do you want to overwrite it?"), self.parent_window):
                    overwrite = True
                else:
                    overwrite = False

            if overwrite or overwrite is None:
                movies = self.db.session.execute(self.get_query())

                writer = csv.writer(open(filename[0], 'w'), dialect=csv.excel)
                # write column header row
                writer.writerow(self.fields_to_export)
                # write data rows
                for movie in movies:
                    t = []
                    for s in self.exported_columns:
                        t.append(movie[s])
                    writer.writerow(t)
                gutils.info(_("%s file has been created.") % "CSV", self.parent_window)



# -*- coding: UTF-8 -*-
# vim: fdm=marker

__revision__ = '$Id$'

# Copyright (c) 2005-2009 Vasco Nunes, Piotr Ożarowski
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

# You may use and distribute this software under the terms of the
# GNU General Public License, version 2 or later

import logging
import os.path

from sqlalchemy.exc import IntegrityError, OperationalError

import db
import gutils
from gettext import gettext as _

log = logging.getLogger("Griffith")

@gutils.popup_message(_('Upgrading database...'))
def upgrade_database(self, version, config):
    """Create new db or update existing one to current format"""
    b = self.engine
    if version == 0 or version is None:
        log.info('Creating new database...')
        # version is 0 or none only for new databases
        db.metadata.create_all(b)
        with b.begin() as conn:
            conn.execute(db.tables.configuration.insert(), {'param': 'version', 'value': str(self.version)})
            for name in (
                'DVD','DVD-R','DVD-RW','DVD+R','DVD+RW','DVD-RAM','CD','CD-RW','VCD','SVCD','VHS','BETACAM','LaserDisc','HD DVD','Blu-ray'
            ):
                conn.execute(db.tables.media.insert(), {'name': name})
            for name in ('16:9','16:10','4:3'):
                conn.execute(db.tables.ratios.insert(), {'name': name})
            for name in ('AC-3 Dolby audio','OGG','MP3','MPEG-1','MPEG-2','AAC','Windows Media Audio'):
                conn.execute(db.tables.acodecs.insert(), {'name': name})
            for name in ('MPEG-1','MPEG-2','XviD','DivX','H.264','RealVideo','QuickTime','Windows Media Video'):
                conn.execute(db.tables.vcodecs.insert(), {'name': name})
            for name in ('mono','stereo','5.1','7.1'):
                conn.execute(db.tables.achannels.insert(), {'name': name})
            for name in (
                'DVD VOB','MPL2 (.txt)','MicroDVD (.sub)','SubRip (.srt)','SubViewer2 (.sub)','Sub Station Alpha (.ssa)','Advanced Sub Station Alpha (.ssa)'
            ):
                conn.execute(db.tables.subformats.insert(), {'name': name})
            for name in (
                _('Brazilian Portuguese'),_('Bulgarian'),_('Catalan'),_('Czech'),_('Danish'),_('Dutch'),_('English'),_('Estonian'),_('French'),_('German'),
                _('Greek'),_('Hungarian'),_('Indonesian'),_('Italian'),_('Japanese'),_('Korean'),_('Norwegian Bokmal'),_('Occitan'),_('Pashto'),_('Polish'),
                _('Portuguese'),_('Russian'),_('Simplified Chinese'),_('Slovak'),_('Spanish'),_('Swedish'),_('Turkish')
            ):
                conn.execute(db.tables.languages.insert(), {'name': name})
            for name in (_('Favourite'), _('Buy me')):
                conn.execute(db.tables.tags.insert(), {'name': name})
        return True  # upgrade process finished
    #
    # next steps are only for existing databases with an outdated structure
    # not for new created databases
    #
    if version == 1:  # fix changes between v1 and v2
        version += 1
        log.info("Upgrading database to version %d...", version)
        with b.begin() as conn:
            conn.exec_driver_sql("UPDATE loans SET return_date='2007-01-01' WHERE return_date='None';")
        db_version = self.session.query(db.Configuration).filter_by(param='version').one()
        db_version.value = str(version)
        self.session.add(db_version)
        self.session.commit()
    if version == 2:    # fix changes between v2 and v3
        # e_type = self.session.bind.engine.dialect.name
        e_type = self.session.bind.name
        version += 1
        log.info("Upgrading database to version %d...", version)

        # create new table
        db.tables.posters.create(checkfirst=True, bind=b)
        db.tables.filters.create(checkfirst=True, bind=b)
        db.tables.ratios.create(checkfirst=True, bind=b)
        try:
            with b.begin() as conn:
                conn.execute(db.tables.ratios.insert(), {'name': '16:9'})
                conn.execute(db.tables.ratios.insert(), {'name': '4:3'})
        except IntegrityError as e:
            # if the following conversion of the posters takes too long and
            # the user kills the application the database is in an undefined
            # state which throws that exception
            log.warning("Cannot add values because they exist already: %s", e)

        log.info('... adding new columns')
        # common SQL statements
        queries = {'poster_md5': 'ALTER TABLE movies ADD poster_md5 VARCHAR(32) NULL REFERENCES posters(md5sum);',
                   'ratio_id': 'ALTER TABLE movies ADD ratio_id INTEGER NULL REFERENCES ratios(ratio_id);',
                   'screenplay': 'ALTER TABLE movies ADD screenplay VARCHAR(256) NULL;',
                   'cameraman': 'ALTER TABLE movies ADD cameraman VARCHAR(256) NULL;'}
        # if needed some db specific SQL statements
        if e_type == 'mysql':
            pass
        elif e_type == 'mssql':
            pass
        for key, query in list(queries.items()):
            try:
                with b.begin() as conn:
                    conn.exec_driver_sql(query)
            except OperationalError as ex:
                if str(ex).lower().find('duplicate column name') > -1:
                    log.warning("Cannot add '%s' column because it exists "
                                "already: %s", key, ex)
                    continue
                else:
                    log.error("Cannot add '%s' column: %s", key, ex)
                    return False
            except Exception as ex:
                log.error("Cannot add '%s' column: %s", key, ex)
                return False

        log.info('... saving posters in database')
        posters_dir = get_old_posters_location(self.data_dir, config,
                                               clean_config=True)
        updated = {}
        movies_table = db.metadata.tables['movies']
        for movie in self.session.query(db.Movie.image).all():
            poster_file_name = os.path.join(posters_dir, "%s.jpg" % movie.image)
            if poster_file_name in updated:
                continue
            if os.path.isfile(poster_file_name):
                with (poster_file_name, 'rb') as fp:
                    poster_md5 = gutils.md5sum(fp)
                    poster = self.session.query(db.Poster).filter_by(
                        md5sum=poster_md5).first()
                    if not poster:
                        fp.seek(0)
                        poster = db.Poster(md5sum=poster_md5, data=fp.read())
                        self.session.add(poster)

                update_query = (
                    movies_table.update()
                    .where(movies_table.c.image == movie.image)
                    .values(poster_md5=poster_md5, image=None)
                )
                # deactivating a 0.12 feature
                save_update_onupdated = movies_table.c['updated'].onupdate
                movies_table.c['updated'].onupdate = None

                try:
                    # yeah, we're committing inside the loop,
                    # it slows down the process a lot, but at least we can skip
                    # buggy posters
                    with b.begin() as conn:
                        conn.execute(update_query)
                    self.session.commit()
                except Exception as e:
                    self.session.rollback()
                    log.error(e)
                else:
                    updated[poster_file_name] = True
                    try:
                        os.remove(poster_file_name)
                    except Exception as ex:
                        log.warning(f"cannot remove {poster_file_name}, "
                                    f"{type(ex).__name__}: {ex}")
                finally:
                    movies_table.c['updated'].onupdate = save_update_onupdated
            else:
                log.warning("file not found: %s", movie.image)
                update_query = (
                    movies_table.update()
                    .where(movies_table.c.image == movie.image)
                    .values(image=None)
                )
                # deactivating a 0.12 feature
                save_update_onupdated = movies_table.c['updated'].onupdate
                movies_table.c['updated'].onupdate = None
                try:
                    with b.begin() as conn:
                        conn.execute(update_query)
                finally:
                    movies_table.c['updated'].onupdate = save_update_onupdated
                updated[poster_file_name] = True
        del updated

        db_version = (self.session.query(db.Configuration)
                      .filter_by(param='version').one())
        db_version.value = str(version)
        self.session.add(db_version)
        self.session.commit()

    if version == 3:    # fix changes between v3 and v4
        version += 1
        log.info("Upgrading database to version %d...", version)

        log.info('... adding new columns')
        # common SQL statements
        queries = {'barcode': 'ALTER TABLE movies ADD barcode VARCHAR(32) NULL;',
                   'width': 'ALTER TABLE movies ADD width SMALLINT NULL;',
                   'height': 'ALTER TABLE movies ADD height SMALLINT NULL;'}

        for key, query in list(queries.items()):
            try:
                with b.begin() as conn:
                    conn.exec_driver_sql(query)
            except Exception as e:
                log.error("Cannot add '%s' column: %s", key, e)
                return False

        db_version = self.session.query(db.Configuration).filter_by(param='version').one()
        db_version.value = str(version)
        self.session.add(db_version)
        self.session.commit()

    if version == 4:    # fix changes between v4 and v5
        version += 1
        log.info("Upgrading database to version %d...", version)

        log.info('... deleting old filters')
        # new format, filters were introduced in -beta, so we'll free to
        # delete them without warning
        query = 'DELETE FROM filters;'
        with b.begin() as conn:
            conn.exec_driver_sql(query)

        db_version = self.session.query(db.Configuration).filter_by(param='version').one()
        db_version.value = str(version)
        self.session.add(db_version)
        self.session.commit()

    if version == 5:    # fix changes between v5 and v6
        version += 1
        log.info("Upgrading database to version %d...", version)

        # common SQL statements
        if self.session.bind.name in ['postgres', 'postgresql']:
            queries = {'created': 'ALTER TABLE movies ADD created TIMESTAMP;',
                       'updated': 'ALTER TABLE movies ADD updated TIMESTAMP;'}
        else:
            queries = {'created': 'ALTER TABLE movies ADD created DATETIME;',
                       'updated': 'ALTER TABLE movies ADD updated DATETIME;'}
        for key, query in list(queries.items()):
            try:
                with b.begin() as conn:
                    conn.exec_driver_sql(query)
            except Exception as e:
                log.error("Cannot add '%s' column: %s", key, e)
                return False

        db_version = self.session.query(db.Configuration).filter_by(param='version').one()
        db_version.value = str(version)
        self.session.add(db_version)
        self.session.commit()

    return True


# ---------------------------------------------------
# for Griffith <= 0.6.2 compatibility
# ---------------------------------------------------


def convert_from_old_db(config, source_file, destination_file, locations):  #{{{
    """
    convert .gri database into .bd one
    """

    log.info('Converting old database - it can take several minutes...')
    log.debug("Source file: %s", source_file)
    gutils.info(_("Griffith will now convert your database to the new format. This can take several minutes if you have a large database."))
    from .sql import GriffithSQL
    from .gutils import digits_only
    import os

    if not os.path.isfile(source_file):
        return False

    if 'home' not in locations:
        log.error("locations doesn't contain home path, cannot convert old database")
        return False

    with open(source_file) as fp:
        db_file_start = fp.readline()[:47]
    if db_file_start == '** This file contains an SQLite 2.1 database **':
        log.debug('SQLite 2.1 detected')
        try:
            import sqlite
        except ImportError:
            log.error('Old DB conversion: please install pysqlite legacy (v1.0)')
            gutils.warning(
                _("Old DB conversion: please install pysqlite legacy (v1.0)"))
            return False
    else:
        try:    # Python 2.5
            from sqlite3 import dbapi2 as sqlite
        except ImportError:  # Python < 2.5 - try to use pysqlite2
            from pysqlite3 import dbapi2 as sqlite

    if os.path.isfile(destination_file):
        # rename destination_file if it already exists
        i = 1
        while True:
            if os.path.isfile("%s_%s" % (destination_file, i)):
                i += 1
            else:
                break
        os.rename(destination_file, "%s_%s" % (destination_file, i))

    try:
        old_db = sqlite.connect(source_file)
    except sqlite.DatabaseError as e:
        if str(e) == 'file is encrypted or is not a database':
            print('Your database is most probably in SQLite2 format, please convert it to SQLite3:')
            print('$ sqlite ~/.griffith/griffith.gri .dump | sqlite3 ~/.griffith/griffith.gri3')
            print('$ mv ~/.griffith/griffith.gri{,2}')
            print('$ mv ~/.griffith/griffith.gri{3,}')
            print('or install pysqlite in version 1.0')
            gutils.warning(_("Your database is most probably in SQLite2 format, please convert it to SQLite3"))
        else:
            raise
        return False

    old_cursor = old_db.cursor()

    # fix old database
    old_cursor.execute('PRAGMA encoding = "UTF-8";')
    old_cursor.execute("UPDATE movies SET media = '1' WHERE media = 'DVD';")
    old_cursor.execute("UPDATE movies SET media = '2' WHERE media = 'DVD-R';")
    old_cursor.execute("UPDATE movies SET media = '3' WHERE media = 'DVD-RW';")
    old_cursor.execute("UPDATE movies SET media = '4' WHERE media = 'DVD+R';")
    old_cursor.execute("UPDATE movies SET media = '5' WHERE media = 'DVD+RW';")
    old_cursor.execute("UPDATE movies SET media = '6' WHERE media = 'DVD-RAM';")
    old_cursor.execute("UPDATE movies SET media = '7' WHERE media = 'DivX';")
    old_cursor.execute("UPDATE movies SET media = '7' WHERE media = 'DIVX';")
    old_cursor.execute("UPDATE movies SET media = '7' WHERE media = 'XviD';")
    old_cursor.execute("UPDATE movies SET media = '7' WHERE media = 'XVID';")
    old_cursor.execute("UPDATE movies SET media = '7' WHERE media = 'WMV';")
    old_cursor.execute("UPDATE movies SET media = '9' WHERE media = 'VCD';")
    old_cursor.execute("UPDATE movies SET media = '10' WHERE media = 'SVCD';")
    old_cursor.execute("UPDATE movies SET media = '11' WHERE media = 'VHS';")
    old_cursor.execute("UPDATE movies SET media = '12' WHERE media = 'BETACAM';")
    old_cursor.execute("UPDATE movies SET collection_id=0 WHERE collection_id<1")
    old_cursor.execute("UPDATE movies SET volume_id=0 WHERE volume_id<1")
    old_cursor.execute("UPDATE movies SET color=NULL WHERE color<1 OR color='' OR color>3")
    old_cursor.execute("UPDATE movies SET condition=NULL WHERE condition<0 OR condition='' OR condition>5")
    old_cursor.execute("UPDATE movies SET layers=NULL WHERE layers<0 OR layers='' OR layers>4")
    old_cursor.execute("UPDATE movies SET region=NULL WHERE region='' OR region=2 OR region<0 OR region>8")
    old_cursor.execute("UPDATE movies SET year=NULL WHERE year<1900 or year>2007")
    old_cursor.execute("UPDATE movies SET rating = 0 WHERE rating NOT IN (0,"
                       "1,2,3,4,5,6,7,8,9,10);")  # rating>10 doesn't work with some DB
    old_cursor.execute("UPDATE movies SET runtime = NULL WHERE runtime > "
                       "10000;")  # remove strings
    old_cursor.execute("UPDATE loans SET return_date=NULL WHERE return_date=''")
    old_cursor.execute("UPDATE loans SET return_date=NULL WHERE return_date='None'")
    old_cursor.execute("DELETE FROM loans WHERE date='' OR date ISNULL")
    old_cursor.execute("DELETE FROM volumes WHERE name = ''")
    old_cursor.execute("DELETE FROM volumes WHERE name = 'None'")
    old_cursor.execute("DELETE FROM collections WHERE name = ''")
    old_cursor.execute("DELETE FROM collections WHERE name = 'None'")
    old_cursor.execute("DELETE FROM languages WHERE name = ''")

    config.set('type', 'sqlite', section='database')
    config.set('file', 'griffith.db', section='database')
    config['posters'] = 'posters'
    config.set('color', 0, section='defaults')
    config.set('condition', 0, section='defaults')
    config.set('layers', 0, section='defaults')
    config.set('media', 0, section='defaults')
    config.set('region', 0, section='defaults')
    config.set('vcodec', 0, section='defaults')
    locations['posters'] = os.path.join(locations['home'], 'posters')
    new_db = GriffithSQL(config, locations['home'], fallback=False)

    # collections
    collection_mapper = {'': None, 0: None, '0': None, -1: None, '-1': None}
    # loaned status will be set later - buggy databases :-(
    old_cursor.execute("SELECT id, name FROM collections;")
    for i in old_cursor.fetchall():
        o = db.Collection(name=i[1])
        try:
            new_db.session.add(o)
            new_db.session.commit()
        except Exception as e:
            log.error(e)
            continue
        collection_mapper[i[0]] = o.collection_id

    # volumes
    volume_mapper = {'': None, 0: None, '0': None, -1: None,
                     '-1': None}
    # loaned status will be set later - buggy databases :-(
    old_cursor.execute("SELECT id, name FROM volumes;")
    for i in old_cursor.fetchall():
        o = db.Volume(name=i[1])
        try:
            new_db.session.add(o)
            new_db.session.commit()
        except Exception as e:
            log.error(e)
            continue
        volume_mapper[i[0]] = o.volume_id

    # people
    person_mapper = {}
    old_cursor.execute("SELECT id, name, email, phone FROM people;")
    for i in old_cursor.fetchall():
        o = db.Person(name=i[1], email=i[2], phone=i[3])
        try:
            new_db.session.add(o)
            new_db.session.commit()
        except Exception as e:
            log.error(e)
            continue
        person_mapper[i[0]] = o.person_id

    # languages
    language_mapper = {'': None, 0: None, '0': None, -1: None, '-1': None}
    old_cursor.execute("SELECT id, name FROM languages;")
    for i in old_cursor.fetchall():
        o = new_db.session.query(db.Lang).filter_by(name=i[1]).first()
        if o is not None:
            language_mapper[i[0]] = o.lang_id
        else:
            o = db.Lang(name=i[1])
            try:
                new_db.session.add(o)
                new_db.session.commit()
            except Exception as e:
                log.error(e)
                continue
            language_mapper[i[0]] = o.lang_id

    # media
    # FIXME: what may have been the intention for the two "''" keys, a key is
    #  unique in a dictionary?
    medium_mapper = {'': None, 0: None, '0': None, -1: None, '-1': None}
    old_cursor.execute("SELECT id, name FROM media;")
    for i in old_cursor.fetchall():
        o = new_db.session.query(db.Medium).filter_by(name=i[1]).first()
        if o is not None:
            medium_mapper[i[0]] = o.medium_id
        else:
            o = db.Medium(name=i[1])
            try:
                new_db.session.add(o)
                new_db.session.commit()
            except Exception as e:
                log.error(e)
                continue
            medium_mapper[i[0]] = o.medium_id

    # tags
    tag_mapper = {}
    old_cursor.execute("SELECT id, name FROM tags;")
    for i in old_cursor.fetchall():
        o = new_db.session.query(db.Tag).filter_by(name=i[1]).first()
        if o is not None:
            tag_mapper[i[0]] = o.tag_id
        else:
            o = db.Tag(name=i[1])
            try:
                new_db.session.add(o)
                new_db.session.commit()
            except Exception as e:
                log.error(e)
                continue
            tag_mapper[i[0]] = o.tag_id

    # movies
    movie_mapper = {}
    old_cursor.execute("""
        SELECT id, volume_id, collection_id, original_title, title, director,
            number, image, plot, country, year, runtime, classification,
            genre, studio, site, imdb, actors, trailer, rating, loaned,
            media, num_media, obs, seen, region, condition, color, layers
        FROM movies ORDER BY number;""")
    for i in old_cursor.fetchall():
        o = db.Movie()
        o.number = digits_only(i[6])
        if i[1] in volume_mapper:
            o.volume_id = volume_mapper[i[1]]
        if i[2] in collection_mapper:
            o.collection_id = collection_mapper[i[2]]
        o.o_title = i[3][:255]
        o.title = i[4][:255]
        o.director = i[5][:255]
        o.image = i[7][:128]
        o.plot = i[8]
        o.country = i[9][:128]
        o.year = digits_only(i[10])
        o.runtime = digits_only(i[11])
        o.classification = i[12][:128]
        o.genre = i[13][:128]
        o.studio = i[14][:128]
        o.o_site = i[15][:255]
        o.site = i[16][:255]
        o.cast = i[17]
        o.trailer = i[18][:255]
        o.rating = digits_only(i[19])
        # o.loaned = bool(i[20])  # updated later
        if int(i[21]) in medium_mapper:
            o.medium_id = medium_mapper[int(i[21])]
        o.media_num = digits_only(i[22])
        o.notes = i[23]
        o.seen = bool(i[24])
        o.region = digits_only(i[25])
        o.cond = digits_only(i[26], 5)
        o.color = digits_only(i[27], 3)
        o.layers = digits_only(i[28], 4)

        try:
            new_db.session.add(o)
            new_db.session.commit()
        except Exception as e:
            log.error(e)
            continue
        movie_mapper[i[0]] = o.movie_id

    # movie tag
    old_cursor.execute("SELECT movie_id, tag_id FROM movie_tag WHERE movie_id IN (SELECT id FROM movies);")
    for i in old_cursor.fetchall():
        o = new_db.session.query(db.MovieTag).filter_by(movie_id=movie_mapper[i[0]], tag_id=tag_mapper[i[1]]).first()
        if o is None:
            m = new_db.session.query(db.Movie).filter_by(movie_id=movie_mapper[i[0]]).one()
            t = new_db.session.query(db.Tag).filter_by(tag_id=tag_mapper[i[1]]).one()
            m.tags.append(t)
            try:
                new_db.session.add(m)
                new_db.session.commit()
            except Exception as e:
                log.error(e)
                continue

    # movie lang
    old_cursor.execute("SELECT movie_id, lang_id, type FROM movie_lang WHERE movie_id IN (SELECT id FROM movies);")
    for i in old_cursor.fetchall():
        o = new_db.session.query(db.MovieLang).filter_by(
            movie_id=movie_mapper[i[0]], lang_id=language_mapper[i[1]],
            movie_type=i[2]).first()
        if o is None:
            m = new_db.session.query(db.Movie).filter_by(movie_id=movie_mapper[i[0]]).one()
            lang = db.MovieLang(lang_id=language_mapper[i[1]], movie_type=i[2])
            m.languages.append(lang)
            try:
                new_db.session.add(m)
                new_db.session.commit()
            except Exception as e:
                log.error(e)
                continue

    # loans
    old_cursor.execute("SELECT person_id, movie_id, volume_id, collection_id, date, return_date FROM loans;")
    for i in old_cursor.fetchall():
        vol = col = None
        not_returned = i[5] is None

        if int(i[2]) > 0:
            try:
                vol = new_db.session.query(db.Volume).filter_by(volume_id=volume_mapper[i[2]]).one()
            except Exception as e:
                log.error(e)
                continue
        if int(i[3]) > 0:
            try:
                col = new_db.session.query(db.Collection).filter_by(collection_id=collection_mapper[i[3]]).one()
            except Exception as e:
                log.error(e)
                continue
        if int(i[1]) == 0:
            if vol is not None and len(vol.movies) > 0:
                m = vol.movies[0]
            elif col is not None and len(col.movies) > 0:
                m = col.movies[0]
            else:
                log.warning("Cannot find associated movie for this loan (%s)"
                            % i)
                continue
        else:
            try:
                m = new_db.session.query(db.Movie).filter_by(movie_id=movie_mapper[i[1]]).one()
            except Exception as e:
                log.error(e)
                continue

        lang = db.Loan()
        lang.person_id = person_mapper[i[0]]
        lang.date = str(i[4])[:10]
        if not_returned:
            m.loaned = True
            lang.return_date = None
        else:
            lang.return_date = str(i[5])[:10]

        # update volume / collection status
        if int(i[2]) > 0:
            lang.volume_id = volume_mapper[i[2]]
            if not_returned:
                vol.loaned = True
                vol.save()
        if int(i[3]) > 0:
            lang.collection_id = collection_mapper[i[3]]
            if not_returned:
                col.loaned = True
                col.save()
        lang.save()
        m.loans.append(lang)
        try:
            new_db.session.add(m)
            new_db.session.commit()
        except Exception as e:
            log.error(e)
            continue
    # clear_mappers()
    return new_db
#}}}


def get_old_posters_location(home_dir, config, clean_config=False):
    if config.get('posters', None) is not None:
        dirname = config.get('posters')
    elif config.get('type', 'sqlite', section='database') == 'sqlite':
        dbname = config.get('name', 'griffith', section='database')
        if dbname != 'griffith':
            dirname = "posters_sqlite_%s" % dbname
        else:
            dirname = 'posters'
    else:
        dirname = "posters_%(type)s_%(host)s_%(port)s_%(name)s_%(user)s" % config.to_dict('database')
    if clean_config:
        config.remove_option('posters')
        config.save()
    return os.path.join(home_dir, dirname)



# -*- coding: UTF-8 -*-

__revision__ = '$Id$'

# Copyright © 2009 Piotr Ożarowski
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published byp
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

# You may use and distribute this software under the terms of the
# GNU General Public License, version 2 or later

import logging
from subprocess import Popen

from gutils import is_windows_system
from plugins.extensions import GriffithExtensionBase as Base
from gettext import gettext as _

log = logging.getLogger('Griffith')


class GriffithExtension(Base):
    name = 'Player'
    description = _('Plays movie trailer')
    author = 'Piotr Ożarowski'
    email = 'piotr@griffith.cc'
    version = 1
    api = 1
    enabled = False  # disabled by default

    preferences = {'command': {'name': _('Command'),
                               'hint': _('{1} (if given) will be replaced with file path'),
                               'default': 'mplayer {1}',
                               'type': str}}
    if is_windows_system():
        preferences['command']['default'] = ''

    toolbar_icon = 'ge_player.png'

    def toolbar_icon_clicked(self, widget, movie):
        if not movie or not movie.trailer:
            return False

        use_shell = True
        command = self.get_config_value('command', self.preferences['command']['default'])
        if is_windows_system():
            use_shell = False  # Popen with shell=True doesn't work under
            # windows with spaces in filenames
            if not command:
                import win32api
                log.debug('try ShellExecute with trailer %s' % movie.trailer)
                win32api.ShellExecute(0, None, movie.trailer, None, None, 0)
                return

        if '{1}' in command:
            command = command.replace('{1}', movie.trailer)
        else:
            # make a sequence results in Popen calls list2cmdline
            command = [command, movie.trailer]

        log.debug(command)
        Popen(command, shell=use_shell)

    def maintree_clicked(self, selection, movie):
        if movie and movie.trailer:
            self.toolbar_icon_widget.set_sensitive(True)
        else:
            self.toolbar_icon_widget.set_sensitive(False)



# -*- coding: utf-8 -*-

__revision__ = '$Id$'

# Copyright (c) 2005-2007 Piotr Ożarowski
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

# You may use and distribute this software under the terms of the
# GNU General Public License, version 2 or later

import gutils
import movie
from gettext import gettext as _

plugin_name         = 'Tanuki-Anime'
plugin_description  = 'Największy zbiór recenzji anime w Polsce'
plugin_url          = 'anime.tanuki.pl'
plugin_language     = _('Polish')
plugin_author       = 'Piotr Ozarowski'
plugin_author_email = '<ozarow+griffith@gmail.com>'
plugin_version      = '1.3'


class Plugin(movie.Movie):
    def __init__(self, id):
        if str(id).find('http://') != -1:
            self.movie_id = 'TA'
            self.url = str(id)
        else:
            self.movie_id = str(id)
            self.url = "http://anime.tanuki.pl/strony/anime/%s" % str(id)
        self.encode='UTF-8'

    def initialize(self):
        if self.movie_id == 'TA':
            self.movie_id = gutils.trim(self.page, '"><a href="/strony/anime/', '/oceny')
            self.url = "http://anime.tanuki.pl/strony/anime/%s" % self.movie_id

    def get_image(self):
        self.image_url = gutils.trim(self.page, '<img src="/screens/', '<br />')
        self.image_url = gutils.before(self.image_url, '"')
        self.image_url = "http://anime.tanuki.pl/screens/%s" % self.image_url

    def get_o_title(self):
        self.o_title = gutils.trim(self.page, '<h3 class="animename"', '</h3>')
        self.o_title = gutils.after(self.o_title, '>')

    def get_title(self):
        self.title = self.o_title

    def get_director(self):
        self.director = gutils.trim(self.page, "<th scope=\"row\">Reżyser:</th>\n\t<td>", '</td>')

    def get_plot(self):
        self.plot = gutils.trim(self.page, "<div class=\"copycat\">\n", '</div>')

    def get_year(self):
        self.year = gutils.trim(self.page,'<div class=\"sitem\">Rok wydania: <', '</a>')
        self.year = gutils.after(self.year, '>')

    def get_runtime(self):
        self.runtime = gutils.trim(self.page, "<div class=\"sitem\">Czas trwania: <b>\n\t\t", '\n</b>')
        if self.runtime.find('?') != -1:
            self.runtime = ''
        else:
            self.runtime = gutils.after(self.runtime, '×')
            self.runtime = gutils.before(self.runtime, ' min')

    def get_genre(self):
        self.genre = gutils.trim(self.page, "<div class=\"sitem\">Gatunki:\n\t\t", '</div>')
        self.genre = self.genre.replace("\t\t", ' ')

    def get_cast(self):
        self.cast = ''

    def get_classification(self):
        self.classification = ''

    def get_studio(self):
        self.studio = gutils.trim(self.page, "<th scope=\"row\">Studio:</th>\n\t<td>\n", "\t</td>")

    def get_o_site(self):
        self.o_site = ''

    def get_site(self):
        self.site = self.url

    def get_trailer(self):
        self.trailer = ''

    def get_country(self):
        self.country = ''

    def get_rating(self):
        self.rating = gutils.trim(self.page, ' alt="Ocena ', '/10')

    def get_notes(self):
        self.notes = "Czas trwania: " + gutils.trim(self.page,"<div class=\"sitem\">Czas trwania: <b>\n\t\t","\n</b>") + '\n'

        t = self.page.find("<tr><th scope=\"row\">Autor:</th>")
        if t != -1:
            self.notes += "Autor: %s\n" % gutils.trim(self.page[t:], "<td>\n", "\t</td>")

        t = self.page.find("<th scope=\"row\">Projekt:</th>")
        if t != -1:
            self.notes += "Projekt: %s\n" % gutils.trim(self.page[t:], "<td>\n", "\t</td>")

        t = self.page.find("<tr><th scope=\"row\">Scenariusz:</th>")
        if t != -1:
            self.notes += "Scenariusz: %s\n" % gutils.trim(self.page[t:], "<td>\n", "\t</td>")

        t = self.page.find("<th scope=\"row\">Muzyka:</th>")
        if t != -1:
            self.notes += "Muzyka: %s\n" % gutils.trim(self.page[t:], "<td>\n", "\t</td>")

        self.notes += "\n%s"  % gutils.trim(self.page,"<p class=\"dwazdania\">\n\t\t", "\n</p>")

class SearchPlugin(movie.SearchMovie):
    def __init__(self):
        self.encode='UTF-8'
        self.original_url_search   = 'http://anime.tanuki.pl/strony/anime/lista/title/1/?&title='
        self.translated_url_search = 'http://anime.tanuki.pl/strony/anime/lista/title/1/?&title='

    def search(self,parent_window):
        if not self.open_search(parent_window):
            return None
        tmp = self.page.find("<table class=\"animelist strippedlist\"")
        if tmp == -1:    # only one match!
            self.page = ''
        else:        # multiple matches
            self.page = gutils.trim(self.page,"<table class=\"animelist strippedlist\"", "</tbody>")
            self.page = gutils.after(self.page, "<tbody>")
        return self.page

    def get_searches(self):
        if self.page == '':    # immediately redirection to movie page
            self.number_results = 1
            self.ids.append(self.url)
            self.titles.append(gutils.convert_entities(self.title))
        else:            # multiple matches
            elements = self.page.split("<tr")
            self.number_results = elements[-1]
            if (elements[0]!=''):
                for element in elements:
                    self.ids.append(gutils.trim(element, 'href="/strony/anime/', '" >'))
                    element = gutils.after(element," >\n\t")
                    element = element.replace("</a>\n\t</td>\n", " (")
                    element = element.replace("\t<td>", "")
                    element = element.replace("</td>\n", "; ")
                    element = element.replace("</tr>", "")
                    element = element[:len(element)-3] + ')'
                    self.titles.append(element)
            else:
                self.number_results = 0



# -*- coding: UTF-8 -*-
# vim: fdm=marker
__revision__ = '$Id$'

# Copyright © 2011
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

# You may use and distribute this software under the terms of the
# GNU General Public License, version 2 or later

import logging

#wf from sqlalchemy.orm.interfaces import AttributeExtension
from sqlalchemy.ext.instrumentation import InstrumentationManager
from sqlalchemy.orm import ColumnProperty
from sqlalchemy.types import String

from sqlalchemy import event

log = logging.getLogger('Griffith')


class InstallValidatorListeners(InstrumentationManager):
    def post_configure_attribute(self, class_, key, inst):
        """Add validators for any attributes that can be validated."""
        # SQLAlchemy >= 0.7 (using events)
        prop = inst.prop
        # Only interested in simple columns, not relations
        if isinstance(prop, ColumnProperty) and len(prop.columns) == 1:
            col = prop.columns[0]
            # if we have string column with a length, install a length validator
            if isinstance(col.type, String) and col.type.length:
                event.listen(
                    inst, 'set', LengthValidator(
                        col.name, col.type.length).set, retval=True)


class ValidationError(Exception):
    pass


#wf class LengthValidator(AttributeExtension):
class LengthValidator:
    def __init__(self, name, max_length):
        self.name = name
        self.max_length = max_length

    def set(self, _state, value, _oldvalue, _initiator):
        if value and len(value) > self.max_length:
            # can be changed so that an exception is raised which can be shown
            # in UI but at the moment an exception is silently lost, only
            # written to console
            #raise ValidationError("Length %d exceeds allowed %d for %s" %
            #                    (len(value), self.max_length, self.name))
            log.warning("Length %d exceeds allowed %d for %s; truncating "
                        "value" % (len(value), self.max_length, self.name))
            return value[0:self.max_length]
        return value
